import numpy as np
import shutil
import os
import csv
import pandas as pd
import sys
import sqlite3
import subprocess
import json
import datetime
import re
from T2GEORES import formats as formats
from iapws import IAPWS97

def write_PT_from_t2output(input_dictionary):
	"""It writes the parameter for every block from every well from the last output file of TOUGH2 simulation

	Parameters
	----------
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path', list of wells under the keywords 'WELLS', 'MAKE_UP_WELLS' and 'NOT_PRODUCING_WELL'

	Returns
	-------
	file
	  {well_name}_PT.txt: file containing the information for every block from every well

	Attention
	---------
	The layers and well block need  to be on the databse

	Examples
	--------
	>>> write_PT_from_t2output(input_dictionary)
	"""

	wells=[]

	for key in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	db_path=input_dictionary['db_path']

	t2_output_file="../model/t2/t2.out"
	if os.path.isfile(t2_output_file):
		pass
	else:
		return "Theres is not t2.out file on t2/t2.out"

	#Extracts all the times the line 'OUTPUT DATA AFTER' was printed
	output_headers=[]
	with open(t2_output_file,'r') as t2_file:
		t2_output_array = t2_file.readlines()
		for line_i, line in enumerate(t2_output_array):
			if "OUTPUT DATA AFTER" in line.rstrip():
				output_headers.append(line_i)

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	#Create a dictionary containing all the blocks in a well

	blocks_wells={}
	wells_data={}

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY  middle DESC;",conn)
	for name in sorted(wells):
		wells_data[name]=""
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in data_layer['correlative'].values:
				blocks_wells[n+data_block['blockcorr'].values[0]]=name
	
	#Select the last line of TOUGH2 output file
	for line in t2_output_array[output_headers[-1]:-1]:
		for block in blocks_wells:
			if  block in line.split() and len(line.split())==12: #11
				wells_data[blocks_wells[block]]+="%s\n"%(','.join(line.split()))

	#Writes an output file for every well
	for well in wells_data:
		file_out=open("../output/PT/txt/%s_PT.dat"%(well), "w")
		file_out.write("ELEM,INDEX,P,T,SG,SW,X(WAT1),X(WAT2),PCAP,DG,DW,LOG(PERM)\n")
		file_out.write(wells_data[well])
		file_out.close()

	conn.close()
	
def from_sav_to_json(sav_version='sav1'):
	"""It writes a json file with temperature and pressure data from the specified .sav for every block including coordinates
	
	Parameters
	----------
	sav_version : str
	  Extension of sav file, i.e. sav, sav1, sav2, etc.

	Returns
	-------
	file
	  PT_json_from_sav.txt : on  ../output/PT/json/

	Attention
	---------
	The ELEME.json file needs to be updated

	Examples
	--------
	>>> from_sav_to_json(sav_version='sav1')
	"""

	output_sav_file="../model/t2/%s"%sav_version

	#Generates a dictionary with block as keyword and x,y,z coordinates
	with open('../mesh/ELEME.json') as file:
	  	blocks_position=json.load(file)

	eleme_dict={}
	for block in blocks_position:
		eleme_dict[block]=[blocks_position[block]['X'],blocks_position[block]['Y'],blocks_position[block]['Z']]

	#Creates a string from .sav file
	savfile=open(output_sav_file,"r")
	savstring=[]
	for linesav in savfile:
		savstring.append(linesav.rstrip())
	savfile.close()

	#Stores Pressure and Temperature on the dictionary
	if os.path.isfile(output_sav_file):
		t2_sav_file=open(output_sav_file, "r")
		contidion_found=False
		for t2_sav_line in t2_sav_file:
			if  t2_sav_line[0:5] in  eleme_dict.keys():
				contidion_found=True
				block=t2_sav_line[0:5]
				continue
			if contidion_found:
				eleme_dict[block].extend([float(i) for i in t2_sav_line.split()])
				contidion_found=False
		t2_sav_file.close()

	eleme_pd=pd.DataFrame.from_dict(eleme_dict,orient='index', columns=['X', 'Y','Z','P','T'])
	eleme_pd.to_json("../output/PT/json/PT_json_from_sav.json",orient="index",indent=2)

def PTjson_to_sqlite(input_dictionary,source='t2',):
	"""It stores the defined json file into the database on the table t2PTout
	
	Parameters
	----------
	source : str
	  It can be 't2' or 'sav'
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path'.

	Examples
	--------
	>>> PTjson_to_sqlite("t2")
	"""

	db_path=input_dictionary['db_path']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	if source=="t2":
		if os.path.isfile("../output/PT/json/PT_json.txt"):
			with open("../output/PT/json/PT_json.txt") as f:
				data=json.load(f)
			for element in sorted(data):
				try:
					q="INSERT INTO t2PTout(blockcorr,x,y,z,'index',P,T,SG,SW,X1,X2,PCAP,DG,DW) \
					VALUES ('%s',%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)"%(element,	data[element]['X'],data[element]['Y'],data[element]['Z'],data[element]['INDEX'],\
																			data[element]['P'],data[element]['T'],data[element]['SG'],\
																			data[element]['SW'],data[element]['X(WAT1)'],data[element]['X(WAT2)'],\
																			data[element]['PCAP'],data[element]['DG'],data[element]['DW'])

					c.execute(q)
				except sqlite3.IntegrityError:
					q="UPDATE t2PTout SET \
					x=%s, \
					y=%s, \
					z=%s, \
					'index'=%s, \
					P=%s, \
					T=%s, \
					SG=%s, \
					SW=%s, \
					X1=%s, \
					X2=%s, \
					PCAP=%s, \
					DG=%s, \
					DW=%s \
					WHERE blockcorr='%s'"%(	data[element]['X'],data[element]['Y'],data[element]['Z'],data[element]['INDEX'],\
											data[element]['P'],data[element]['T'],data[element]['SG'],\
											data[element]['SW'],data[element]['X(WAT1)'],data[element]['X(WAT2)'],\
											data[element]['PCAP'],data[element]['DG'],data[element]['DW'],element)
					c.execute(q)
				conn.commit()
	elif source=="sav":
		if os.path.isfile("../output/PT/json/PT_json_from_sav.txt"):
			with open("../output/PT/json/PT_json_from_sav.txt") as f:
				data=json.load(f)
			for element in sorted(data):
				try:
					q="INSERT INTO t2PTout(blockcorr,x,y,z,P,T) \
					VALUES ('%s',%s,%s,%s,%s,%s)"%(element,\
												   data[element]['X'],data[element]['Y'],data[element]['Z'],\
												   data[element]['P'],data[element]['T'])
					c.execute(q)
				except sqlite3.IntegrityError:
					q="UPDATE t2PTout SET \
					x=%s, \
					y=%s, \
					z=%s, \
					P=%s, \
					T=%s \
					WHERE blockcorr='%s'"%(data[element]['ELEM'],\
										   data[element]['X'],data[element]['Y'],data[element]['Z'],\
										   data[element]['P'],data[element]['T'],data[element]['ELEM'])
					c.execute(q)
				conn.commit()

	conn.close()

def write_PT_of_wells_from_t2output_in_time(input_dictionary):
	"""It generates file containing the evolution of every block on every well

	Extrae la evolucion de los bloques relacionados de todos los pozos en la direccion "../output/PT/evol"
	
	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary contaning the path and name of database on keyword 'db_path', list of wells under the keywords 'WELLS', 'MAKE_UP_WELLS' and 'NOT_PRODUCING_WELL'

	Returns
	-------
	file
	  {well}_PT_{layer}.txt : on  ../output/PT/evol

	Examples
	--------
	>>> write_PT_of_wells_from_t2output_in_time(input_dictionary)
	"""

	db_path=input_dictionary['db_path']

	wells=[]

	for key in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	blocks_wells={}
	blocks_data={}

	#Generates a dictionary with the blocks on a well as a keyword 
	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY  middle DESC;",conn)
	for name in sorted(wells):
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in data_layer['correlative'].values:
				blocks_wells[n+data_block['blockcorr'].values[0]]=name
				blocks_data[n+data_block['blockcorr'].values[0]]=""

	#Explore the TOUGH2 output file line by line and store the information if the block list generated on the previous step is on the line.
	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
			if len(t2_line.split())==11 and t2_line.split()[0] in blocks_data.keys():
				t2_array=t2_line.rstrip().split(" ")
				data_list= list(filter(None, t2_array))
				data_list.append(time)
				blocks_data[t2_line.split()[0]]+=','.join(data_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	#Store the data from the dictionary in files
	for block in blocks_data:
		evol_file_out=open("../output/PT/evol/%s_PT_%s_evol.dat"%(blocks_wells[block],block[0]), "w")
		evol_file_out.write("ELEM,INDEX,P,T,SG,SW,X(WAT1),X(WAT2),PCAP,DG,DW,TIME\n")
		evol_file_out.write(blocks_data[block])
		evol_file_out.close()	
	conn.close()

def gen_evol(input_dictionary):
	"""It generates an output file containing flow and flowing enthalpy for each GEN element. As a convention the library it is suggested to use GEN for any source/sink that is not a well

	Parameters
	----------
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path'.

	Returns
	-------
	file
	  {GEN}_{BLOCK}_{NICKNAME}.txt : on  ../output/mh/txt

	"""

	db_path=input_dictionary['db_path']

	#List the GEN elements
	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	data_source=pd.read_sql_query("SELECT well,blockcorr,source_nickname FROM t2wellsource WHERE  source_nickname LIKE'GEN*' ORDER BY source_nickname;",conn)

	final_t2=""
	output_fi_file="../model/t2/t2.out"

	#Initialize a dictionary containing the file path and name.
	dictionary_files={}
	for n in range(len(data_source)):
		well=data_source['well'][n]
		blockcorr=data_source['blockcorr'][n]
		source=data_source['source_nickname'][n]
		dictionary_files[well]={'filename':"../output/mh/txt/%s_%s_%s_evol_mh.dat"%(well,blockcorr,source),'file_container':"",'blockcorr':blockcorr,'source':source}
		dictionary_files[well]['file_container']+="ELEMENT,SOURCEINDEX,GENERATION RATE,ENTHALPY,X1,X2,FF(GAS),FF(AQ.),P(WB),TIME\n"

	#It reads the TOUGH2 output file line by line and store the data from each GEN element
	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
			for well in dictionary_files:
				if dictionary_files[well]['blockcorr'] in t2_line and dictionary_files[well]['source'] in t2_line:
					t2_array=t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list.append(time)
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	#Creates a file for every GEN elemen
	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()

def src_evol(input_dictionary):
	"""It generates an output file containing flow and flowing enthalpy for each SRC element. As a convention the library it is suggested to use SRC for any source/sink that is a well

	Parameters
	----------
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path'.

	Returns
	-------
	file
	  {GEN}_{BLOCK}_{NICKNAME}.txt : on  ../output/mh/txt

	"""

	#See comment for gen_evol

	db_path=input_dictionary['db_path']

	conn=sqlite3.connect(input_data['db_path'])
	c=conn.cursor()
	data_source=pd.read_sql_query("SELECT well,blockcorr,source_nickname FROM t2wellsource WHERE  source_nickname NOT LIKE'GEN*' ORDER BY source_nickname;",conn)

	final_t2=""
	output_fi_file="../model/t2/t2.out"

	dictionary_files={}

	for n in range(len(data_source)):
		well=data_source['well'][n]
		blockcorr=data_source['blockcorr'][n]
		source=data_source['source_nickname'][n]
		dictionary_files[well]={'filename':"../output/mh/txt/%s_%s_%s_evol_mh.dat"%(well,blockcorr,source),'file_container':"",'blockcorr':blockcorr,'source':source}
		dictionary_files[well]['file_container']+="ELEMENT,SOURCEINDEX,GENERATION RATE,ENTHALPY,X1,X2,FF(GAS),FF(AQ.),P(WB),TIME\n"

	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
			for well in dictionary_files:
				if dictionary_files[well]['blockcorr'] in t2_line and dictionary_files[well]['source'] in t2_line:
					t2_array=t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list.append(time)
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()

def write_PT_from_t2output_from_prod(input_dictionary,sav_version='sav1'):
	"""It writes a pressure and temperature comming from block on every well in the specified .sav file.

	Parameters
	----------
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path'.
	sav_version : str
	  Extension of sav file, i.e. sav, sav1, sav2, etc.

	Returns
	-------
	file
	  {well_name}_PT.txt: it contains the pressure and temperature information for every well

	"""

	db_path=input_dictionary['db_path']

	wells=[]

	for key in ['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	#Listing all the wells and assigning them to a well

	blocks_wells={}
	wells_data={}
	blocks_dictionary={}

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY  middle DESC;",conn)

	for name in sorted(wells):
		wells_data[name]=""
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in data_layer['correlative'].values:
				blocks_wells[n+data_block['blockcorr'].values[0]]=name

	
	output_sav_file="../model/t2/t2.%s"%sav_version
	if os.path.isfile(output_sav_file):
		t2_sav_file=open(output_sav_file, "r")
		contidion_found=False
		for t2_sav_line in t2_sav_file:
			if  t2_sav_line[0:5] in  blocks_wells.keys():
				contidion_found=True
				well=blocks_wells[t2_sav_line[0:5]]
				block=t2_sav_line[0:5]
				continue
			if contidion_found:
				wells_data[well]+="%s%s"%(block,','.join(t2_sav_line.split(" ")))
				contidion_found=False
		t2_sav_file.close()

	for well in wells_data:
		file_out=open("../output/PT/txt/%s_PT.dat"%(well), "w")
		file_out.write("ELEM,P,T\n")
		file_out.write(wells_data[well])
		file_out.close()

	conn.close()

def extract_csv_from_t2out(json_output=False):
	"""It writes the parameter for every block from the last output file of TOUGH2 simulation on csv or json

	Parameters
	----------
	json_output : bool
	  If True a json file is save on ../output/PT/json/

	Returns
	-------
	file
	  PT.csv: on ../output/PT/csv/

	Attention
	---------
	The file ELEME.json needs to be updated
	"""
	
	eleme_dict={}

	ELEME_file='../mesh/ELEME.json'
	if os.path.isfile(ELEME_file):
		with open(ELEME_file) as file:
		  	blocks_position=json.load(file)

		for block in blocks_position:
			eleme_dict[block]=[blocks_position[block]['X'],blocks_position[block]['Y'],blocks_position[block]['Z']]
	else:
		return "The file %s does not exist"%ELEME_file

	last=""
	if os.path.isfile("../model/t2/t2.out"):
		t2file=open("../model/t2/t2.out","r")
	else:
		return "Theres is not t2.out file on t2/t2.out"


	cnt=0
	t2string=[]
	#It finds the last section where OUTPUT DATA AFTER was printed and uses it to know where to start to extract data. But also converts every line of the file into an element in an array
	for linet2 in t2file:
		cnt+=1
		t2string.append(linet2.rstrip())
		if "OUTPUT DATA AFTER" in linet2.rstrip():
			last=linet2.rstrip().split(",")
			line=cnt
	t2file.close()

	high_iteration=[int(s) for s in last[0].split() if s.isdigit()]

	for elementx in eleme_dict:
		cnt2=0
		for lineout in t2string[line+cnt2:-1]:
			if " @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"==lineout:
				cnt2+=1
			elif cnt2>2:
				break
			elif elementx in lineout:
				lineselect=lineout.split()
				eleme_dict[elementx].extend(lineselect)
				break

	csv_columns=['X','Y','Z','ELEM','INDEX','P','T','SG','SW','X(WAT1)','X(WAT2)','PCAP','DG','DW']

	if json_output:
		eleme_pd=pd.DataFrame.from_dict(eleme_dict,orient='index',columns=csv_columns)
		dtype= {'X':'float',
				 'Y':'float',
				 'Z':'float',
				 'ELEM':'str',
				 'INDEX':'float',
				 'P':'float',
				 'T':'float',
				 'SG':'float',
				 'SW':'float',
				 'X(WAT1)':'float',
				 'X(WAT2)':'float',
				 'PCAP':'float',
				 'DG':'float',
				 'DW':'float'}
		eleme_pd= eleme_pd.astype(dtype)
		eleme_pd.to_json("../output/PT/json/PT_json.txt",orient="index",indent=2)

	with open("../output/PT/csv/PT.csv",'w') as file:
		file.write(','.join(csv_columns))
		file.write('\n')
		for key in eleme_dict.keys():
			string=""
			for d in eleme_dict[key]:
				string+="%s,"%(d)
			file.write(string[0:-2])
			file.write('\n')
	file.close()

def t2_to_json(itime=None,save_full=False):
	"""It creates a severals or a single json file from the output file from the TOUGH2 run

	Parameters
	----------
 	itime: float
	  It defines a time at which a the parameters from the blocks are extracted into json file. Must be on the same units as the TOUGH2 output files (days, seconds, etc.)
	save_full: bool
	  If True it creates a single output json file

	Returns
	-------
	files
	  t2_ouput_{time}.json: on ../output/PT/json/evol/
	file
	  t2_output:  on ../output/PT/json/

	Attention
	---------
	When t2_output is saved, the large could be too large for the system to handle it

	Examples
	--------
	>>> t2_to_json()
	"""

	block__json_file='../mesh/ELEME.json'

	if os.path.isfile(block__json_file):
		with open('../mesh/ELEME.json') as file:
		  	blocks=json.load(file)
	else:
		sys.exit("The file %s or directory do not exist, run ELEM_to_json from regeo_mesh"%output_t2_file)		  	


	parameters={0:"P",1:"T",2:"SG",3:"SW",4:"X(WAT1)",5:"X(WAT2)",6:"PCAP",7:"DG",8:"DW"}

	#Creates a json file for every output time on the TOUGH2 output file
	cnt=0
	output_t2_file="../model/t2/t2.out"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "OUTPUT DATA AFTER" in t2_line:
				if cnt!=0:
					if itime==None or (itime!=None and float(time)==float(itime)):
						t2_pd=pd.DataFrame.from_dict(data_dictionary,orient='index')
						t2_pd.to_json('../output/PT/json/evol/t2_output_%s.json'%time,orient="index",indent=2)
						if itime!=None and itime==float(itime):
							break
				cnt+=1
				time=t2_line.rstrip().split(" ")[-2]
				data_dictionary={}
				data_dictionary[time]={}
			if len(t2_line.split())==12 and t2_line.split()[0] in blocks.keys():
				t2_array=t2_line.rstrip().split(" ")
				data_list= list(filter(None, t2_array))
				data_dictionary[time][data_list[0]]={}
				for i in parameters:
					data_dictionary[time][data_list[0]][parameters[i]]=float(data_list[i+2])
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	if save_full:
		#It generates a dictionary with time as key and json file output as value
		src='../output/PT/json/evol/'
		src_files = os.listdir(src)
		files_dictionary={}
		for file_name in src_files:
			time=file_name.split("_")[2].split(".j")[0]
			full_file_name = os.path.join(src, file_name)
			files_dictionary[time]=full_file_name
	
		#It generates a single json output file, for a large model it can be to large for the system memory
		t2_json='../output/PT/json/t2_output.json'
		t2_output_file=open(t2_json,'w')

		t2_output_file.write("{\n")
		for time in sorted(files_dictionary):
			file=open(files_dictionary[time],'r')
			lines=file.readlines()[1:-2]
			for line in lines:
				t2_output_file.write(line)
			t2_output_file.write(",\n")		
		t2_output_file.write("}")
		t2_output_file.close()

def it2COF(input_dictionary):
	"""Extracts the COF value for each block on the OBSERVATION section of the inverse file

	Parameters
	----------
	input_dictionary : dictionary
		Contains the name of the iTOUGH2 file

	Returns
	-------
	file
		COF_PT.json: on the output/PT/json/ folder

	Attention
	---------
	It might changed based on the iTOUGH2 version

	Examples
	--------
	>>> it2COF(input_dictionary)
	"""

	it2_output_file="../model/t2/%s.out"%input_dictionary['iTOUGH2_file']
	if os.path.isfile(it2_output_file):
		pass
	else:
		return "Theres is not %.out file on t2/%s.out"%(input_dictionary['iTOUGH2_file'],input_dictionary['iTOUGH2_file'])

	#First and last line in between the data
	compare1 = " DATASET              DATAPOINTS          MEAN       MEDIAN    STD. DEV.    AVE. DEV.   SKEWNESS   KURTOSIS      M/S    DWA     COF"
	compare2 = " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -"

	#Empty dataframe
	data = pd.DataFrame(columns=["DATASET","DATAPOINTS","MEAN","MEDIAN","STD. DEV.","AVE. DEV.","SKEWNESS","KURTOSIS","M/S","DWA","COF"])

	#Extracts the data based on compare1 and compare2
	output_headers = []
	with open(it2_output_file,'r') as t2_file:
		it2_output_array = t2_file.readlines()
		save = False
		for line_i, line in enumerate(it2_output_array):

			if compare1 in line.rstrip():
				save = True
			elif compare2 in line.rstrip():
				save = False

			if save:
				output_headers.append(it2_output_array[line_i+1])

	#Stores the date into the appropiate column
	output_headers=output_headers[1:-1]
	for values in output_headers:
		data_i = values.split()
		data = data.append({"DATASET":data_i[0],
							"DATAPOINTS":data_i[2],
							"MEAN":data_i[3],
							"MEDIAN":data_i[4],
							"STD. DEV.":data_i[5],
							"AVE. DEV.":data_i[6],
							"SKEWNESS":data_i[7],
							"KURTOSIS":data_i[8],
							"M/S":data_i[9],
							"DWA":data_i[10],
							"COF":data_i[12]},ignore_index = True)


	data.to_json("../output/PT/json/COF_PT.json",indent=2)

def it2DATASET(input_dictionary):
	"""Extracts the OBSERVATION dataset on the the inverse file

	Parameters
	----------
	input_dictionary : dictionary
		Contains the name of the iTOUGH2 file

	Returns
	-------
	file
		it2_PT.json: on the output/PT/json/ folder

	Attention
	---------
	It might changed based on the iTOUGH2 version

	Examples
	--------
	>>> it2DATASET(input_dictionary)
	"""

	it2_output_file="../model/t2/%s.out"%input_dictionary['iTOUGH2_file']
	if os.path.isfile(it2_output_file):
		pass
	else:
		return "Theres is not %s.out file on t2/%s.out"%(input_dictionary['iTOUGH2_file'],input_dictionary['iTOUGH2_file'])

	#First and last line in between the data
	compare1="    # OBSERVATION   AT TIME [sec]     MEASURED     COMPUTED     RESIDUAL     WEIGHT C.O.F [%]    STD. DEV.    Yi      Wi    DWi +/-"
	compare2=" Residual Plots"

	data = pd.DataFrame(columns=["NUMBER","OBSERVATION","TIME","MEASURED","COMPUTED","RESIDUAL","WEIGHT","C.O.F","STD.DEV"])


	#Extracts the data based on compare1 and compare2
	output_headers=[]
	with open(it2_output_file,'r') as t2_file:
		it2_output_array = t2_file.readlines()
		save = False
		for line_i, line in enumerate(it2_output_array):
			if compare1 in line.rstrip():
				save = True
			elif compare2 in line.rstrip():
				save = False

			if save and 'ABS.' not in line.rstrip() :
				output_headers.append(it2_output_array[line_i+1])

	#Stores the date into the corresponding column

	output_headers=output_headers[2:-3] #[::2]

	for values in output_headers:
		data_i = values.split()
		if 'AA' not in data_i[1]:
			data = data.append({"NUMBER":data_i[0],
								"OBSERVATION":data_i[1],
								"TIME":data_i[2],
								"MEASURED":data_i[3],
								"COMPUTED":data_i[4],
								"RESIDUAL":data_i[5],
								"WEIGHT":data_i[6],
								"C.O.F":data_i[7],
								"STD":data_i[8]},ignore_index = True)

	data.to_json("../output/PT/json/it2_PT.json",indent=2)

def it2OBJF(input_dictionary):
	"""Extracts the value of the objective function from the output inverse file and append it with the current time

	Parameters
	----------
	input_dictionary : dictionary
		Contains the name of the iTOUGH2 file

	Returns
	-------
	file
		OBJ.json: on the output/PT/json/ folder

	Attention
	---------
	It might changed based on the iTOUGH2 version

	Examples
	--------
	>>> it2OBJF(input_dictionary)
	"""


	it2_output_file="../model/t2/%s.out"%input_dictionary['iTOUGH2_file']
	if os.path.isfile(it2_output_file):
		pass
	else:
		return "Theres is not %s.out file on t2/%s.out"%(input_dictionary['iTOUGH2_file'],input_dictionary['iTOUGH2_file'])

	#First and last line in between the data

	compare1 = "Objective Function"
	compare2 = "=================================================================================================================================="

	#Extracts the data based on compare1 and compare2
	cnt=0
	output_headers=[]
	with open(it2_output_file,'r') as t2_file:
		it2_output_array = t2_file.readlines()
		save = False
		for line_i, line in enumerate(it2_output_array):

			if compare1 in line.rstrip():
				cnt+=1
				save = True
			elif compare2 in line.rstrip():
				save = False

			if cnt == 2 and save:
				output_headers.append(it2_output_array[line_i+1])
	
	OBJ_file = "../output/PT/json/OBJ.json"

	OBJ = pd.read_json(OBJ_file)

	#Add the current time to the file
	OBJ.loc[len(OBJ.index)] = [datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S'), float(output_headers[2][1:-4].split()[6])]

	OBJ.to_json(OBJ_file,indent=2)


#not documented
def normal_directions(input_dictionary,block,block2):
	"""
	not documented
	"""
	conn=sqlite3.connect(input_dictionary['db_path'])
	points=pd.read_sql_query("SELECT x1,y1,x2,y2,ELEME1,ELEME2 FROM segment WHERE ELEME1='%s' AND ELEME2='%s';"%(block,block2),conn)
	conn.close()
	if block[0]!=block2[0]:
		vectors=[]
		if (formats.formats_t2['LAYERS'][block[0]]-formats.formats_t2['LAYERS'][block2[0]])>0:
			duz=-1
		else:
			duz=1
		dux=0
		duy=0
		points=pd.DataFrame([[dux,duy,duz]],columns=['dux','duy','duz'])
	elif block[0]==block2[0]:
		points['dx']=points['x2']-points['x1']
		points['dy']=points['y2']-points['y1']
		points['r']=(points['dx']**2+points['dy']**2)**0.5
		points['ux']=points['dx']/points['r']
		points['uy']=points['dy']/points['r']
		points['dux']=-points['uy']
		points['duy']=points['ux']
		points['duz']=0
	return points

def output_flows(input_dictionary):
	"""
	Read the .out file from the standard TOUGH2 file and store the data into a database. It includes the model version and timestamp.
	"""

	conn=sqlite3.connect(input_dictionary['db_path'])
	elements=pd.read_sql_query("SELECT DISTINCT ELEME FROM ELEME WHERE model_version=%d;"%(input_dictionary['model_version']),conn)
	elements_list=elements['ELEME'].values.tolist()
	conn.close()

	column_names=['ELEME1', 'ELEME2', 'INDEX','FHEAT','FLOH','FLOF','FLOG','FLOAQ','FLOWTR2','VELG','VELAQ','TURB_COEFF','model_time','model_version','model_output_timestamp']

	time_now=datetime.datetime.now()

	output_t2_file="../model/t2/t2.out"

	bulk_data=[]

	allow=False
	allowed_line=1E50
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for n,t2_line in enumerate(t2_file):
			if "OUTPUT DATA AFTER" in t2_line:
				time=t2_line.rstrip().split(" ")[-2]
				fix_data=[time,input_dictionary['model_version'],time_now]
				allow=True

			if t2_line=="                          (W)        (J/KG)        (KG/S)       (KG/S)       (KG/S)       (KG/S)        (M/S)        (M/S)        (1/M)\n":
				allowed_line=n+2

			if t2_line==" ELEMENT SOURCE INDEX      GENERATION RATE     ENTHALPY      X1           X2          FF(GAS)      FF(AQ.)         P(WB)\n":
				allow=False
				allowed_line=1E50

			if allow and n>=allowed_line:
				content=t2_line.split()
				if len(content)==12 and content[0]!='ELEM1':
					data_list= list(filter(None, content))
					data_list.extend(fix_data)
					bulk_data.append(data_list)

			"""
			Previous logic
			#if len(t2_line.split())==12 and elements.ELEME.str.count(t2_line.split()[0]).sum()==1 and elements.ELEME.str.count(t2_line.split()[1]).sum()==1:
			#if len(t2_line.split())==12 and bool(re.match("^[A-Z][A-Z][0-9][0-9][0-9]$",str(t2_line.split()[0]))) and bool(re.match("^[A-Z][A-Z][0-9][0-9][0-9]$",str(t2_line.split()[1]))): #3000s
			if len(t2_line.split())==12 and len(t2_line.split()[0])==5 and not(any(y in t2_line.split() for y in ['ELEM1','GENER'] )): #2950s #t2_line.split()[0] in elements_list and t2_line.split()[1] in elements_list: #3500s
				#print(t2_line.split())
				t2_array=t2_line.rstrip().split(" ")
				data_list= list(filter(None, t2_array))
				data_list.extend(fix_data)
				data_line={}

				for i,name in enumerate(column_names):
					try:
						data_line[name]=float(data_list[i])
					except (ValueError,TypeError):
						data_line[name]=data_list[i]
				
				flows_df=flows_df.append(data_line,ignore_index=True)
			"""
		flows_df = pd.DataFrame(bulk_data,columns = column_names)
		t2_file.close()

		#Writing the dataframe to the database

		conn=sqlite3.connect(input_dictionary['db_path'])
		flows_df.to_sql('t2FLOWSout',if_exists='append',con=conn,index=False)
		conn.close()

	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

def flow_direction(input_dictionary):
	"""
	not documented
	"""
	conn=sqlite3.connect(input_dictionary['db_path'])
	elements=pd.read_sql_query("SELECT DISTINCT ELEME FROM ELEME WHERE model_version=%d;"%(input_dictionary['model_version']),conn)
	elements_list=elements['ELEME'].values.tolist()
	
	flows_dict={}

	for element in elements_list:

		#print("PRE DIRECTIONS",datetime.datetime.now())
		directions=normal_directions(input_dictionary,element)
		#print("AFTER DIRECTIONS",datetime.datetime.now())
		
		#Direction 1
		#print("PRE query flow",datetime.datetime.now())
		flow_data=pd.read_sql_query("SELECT * FROM t2FLOWSout WHERE ELEME1='%s'AND model_time=-3359500000.0"%element,conn)
		#print("AFTER query flow",datetime.datetime.now())

		flow_data['flow_x']=0
		flow_data['flow_y']=0

		#print("PRE FIRST LOOP",datetime.datetime.now())
		for index, row in flow_data.iterrows(): #horizontal

			if row['ELEME2'][0]!=row['ELEME1'][0]:
				#Establish the direction of the flow
				if (formats.formats_t2['LAYERS'][row['ELEME1'][0]]-formats.formats_t2['LAYERS'][row['ELEME2'][0]])>0:
					uy=-1
				else:
					uy=1

				#It is necessary to check if the right position of ELEME1 and ELEME2 exists
				if element==row['ELEME1']:
					flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2']),"flow_y"]=uy*row['FLOF']
				else:
					flow_data.loc[(flow_data['ELEME1']==row['ELEME2']) & (flow_data['ELEME2']==row['ELEME1']),"flow_y"]=-uy*row['FLOF']

			else:

				ux=float(directions.loc[ (directions['ELEME1']==row['ELEME1']) & (directions['ELEME2']==row['ELEME2'])]['dux'])
				uy=float(directions.loc[ (directions['ELEME1']==row['ELEME1']) & (directions['ELEME2']==row['ELEME2'])]['duy'])
				
				flux=float(flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2'])]['FLOF'])
				
				flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2']),"flow_x"]=ux*flux
				flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2']),"flow_y"]=uy*flux
		#print("AFTER FIRST LOOP",datetime.datetime.now())


		#Direction 2
		flow_data2=pd.read_sql_query("SELECT * FROM t2FLOWSout WHERE ELEME2='%s' AND model_time=-3359500000.0"%element,conn)
		flow_data.append(flow_data2)

		#print("PRE SECOND LOOP",datetime.datetime.now())

		for index, row in flow_data.iterrows(): #horizontal

			if row['ELEME2'][0]!=row['ELEME1'][0]:
				#Establish the direction of the flow
				if (formats.formats_t2['LAYERS'][row['ELEME1'][0]]-formats.formats_t2['LAYERS'][row['ELEME2'][0]])>0:
					uy=-1
				else:
					uy=1

				#It is necessary to check if the right position of ELEME1 and ELEME2 exists
				if element==row['ELEME1']:
					flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2']),"flow_y"]=uy*row['FLOF']
				else:
					flow_data.loc[(flow_data['ELEME1']==row['ELEME2']) & (flow_data['ELEME2']==row['ELEME1']),"flow_y"]=-uy*row['FLOF']

			else:

				ux=float(directions.loc[ (directions['ELEME1']==row['ELEME1']) & (directions['ELEME2']==row['ELEME2'])]['dux'])
				uy=float(directions.loc[ (directions['ELEME1']==row['ELEME1']) & (directions['ELEME2']==row['ELEME2'])]['duy'])
				
				flux=float(flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2'])]['FLOF'])
				
				flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2']),"flow_x"]=ux*flux
				flow_data.loc[(flow_data['ELEME1']==row['ELEME1']) & (flow_data['ELEME2']==row['ELEME2']),"flow_y"]=uy*flux
		#print("AFTER SECOND LOOP",datetime.datetime.now())

		#print("PRE CALCS",datetime.datetime.now())
		sum_flowx=flow_data['flow_x'].sum()
		sum_flowy=flow_data['flow_y'].sum()
		flows_dict[element]=[sum_flowx,sum_flowy,(sum_flowx**2+sum_flowy**2)**0.5]
		#print("AFTER CALCS",datetime.datetime.now())
		#print(element,flows_dict[element])

	flows=pd.DataFrame.from_dict(flows_dict,orient='index',columns=['flow_x','flow_y','flow_mag'])
	#left_side=flow_data.loc[flow_data['ELEME1']==element]
	#right_side=flow_data.loc[flow_data['ELEME2']==element]
	flows.to_csv('directions.csv')
	conn.close()

def flow_direction2(input_dictionary):
	"""
	not documented
	"""
	conn=sqlite3.connect(input_dictionary['db_path'])
	elements=pd.read_sql_query("SELECT DISTINCT ELEME FROM ELEME WHERE model_version=%d;"%(input_dictionary['model_version']),conn)
	elements_list=elements['ELEME'].values.tolist()
	
	flows_dict={}

	flow_data=pd.read_sql_query("SELECT * FROM t2FLOWSout WHERE model_time=-3359500000.0",conn)

	flow_data['x_flow']=flow_data.apply(lambda row : normal_directions(input_dictionary,row['ELEME1'],row['ELEME2'])['dux']*row['FLOF'],axis=1)
	flow_data['y_flow']=flow_data.apply(lambda row : normal_directions(input_dictionary,row['ELEME1'],row['ELEME2'])['duy']*row['FLOF'],axis=1)
	flow_data['z_flow']=flow_data.apply(lambda row : normal_directions(input_dictionary,row['ELEME1'],row['ELEME2'])['duz']*row['FLOF'],axis=1)

	#[Finished in 905.5s]

	elements=pd.read_sql_query("SELECT DISTINCT ELEME FROM ELEME WHERE model_version=%d;"%(input_dictionary['model_version']),conn)
	elements_list=elements['ELEME'].values.tolist()

	bulk_data=[]
	time_now=datetime.datetime.now()
	time=-3359500000.0

	for element in elements_list:
		x=flow_data.loc[flow_data['ELEME1']==element,'x_flow'].sum()-flow_data.loc[flow_data['ELEME2']==element,'x_flow'].sum()
		y=flow_data.loc[flow_data['ELEME1']==element,'y_flow'].sum()-flow_data.loc[flow_data['ELEME2']==element,'y_flow'].sum()
		z=flow_data.loc[flow_data['ELEME1']==element,'z_flow'].sum()-flow_data.loc[flow_data['ELEME2']==element,'z_flow'].sum()
		bulk_data.append([element,x,y,z,time,input_dictionary['model_version'],time_now])


	column_names=['ELEME','FLOF_x','FLOF_y','FLOF_z','model_time','model_version','model_output_timestamp']

	flows_df = pd.DataFrame(bulk_data,columns = column_names)

	#Writing the dataframe into the database

	conn=sqlite3.connect(input_dictionary['db_path'])
	flows_df.to_sql('t2FLOWVectors',if_exists='append',con=conn,index=False)
	conn.close()



def src_csv(input_dictionary, path = None):
	"""It generates an output file containing flow and flowing enthalpy for each GEN element from the T2 .csv output file. As a convention the library it is suggested to use GEN for any source/sink that is not a well

	Parameters
	----------
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path'.

	Returns
	-------
	file
	  {GEN}_{BLOCK}_{NICKNAME}.txt : on  ../output/mh/txt

	"""

	db_path=input_dictionary['db_path']

	#List the GEN elements
	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	data_source=pd.read_sql_query("SELECT well,blockcorr,source_nickname FROM t2wellsource WHERE source_nickname NOT LIKE'GEN*' ORDER BY source_nickname;",conn)

	#It reads the gener.csv output file line by line and store the data from each GEN element
	poss_names = ["../model/t2/t2_gener.csv"]
	poss_names.append(path)
	output_t2_file = None 
	for file in poss_names:
		if os.path.isfile(file):
			output_t2_file = file

	if output_t2_file == None :
		sys.exit("No output file located")
	else:
		#Initialize a dictionary containing the file path and name.
		dictionary_files={}
		for n in range(len(data_source)):
			well=data_source['well'][n]
			blockcorr=data_source['blockcorr'][n]
			source=data_source['source_nickname'][n]
			dictionary_files[source]={'filename':"../output/mh/txt/%s_%s_%s_evol_mh.dat"%(well,blockcorr,source),'file_container':"",'blockcorr':blockcorr,'source':source}

		if os.path.isfile(output_t2_file):
			t2_file=open(output_t2_file, "r")
			for i, t2_line in enumerate(t2_file):
				if i == 0:
					t2_array = t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list = [val  for val in str_list if '(' not in val or ')' not in val]
					str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
					str_list.append("TIME")

					for source in dictionary_files:
						dictionary_files[source]['file_container']+=','.join(str_list)+'\n'

				if "TIME [sec]" in t2_line:
					try:
						time = t2_line.rstrip().split(" ")[3]
					except IndexError:
						time = t2_line.rstrip().split(" ")[2]

				for source in dictionary_files:
					if dictionary_files[source]['blockcorr'] in t2_line and dictionary_files[source]['source'] in t2_line:
						t2_array=t2_line.rstrip().split(" ")
						str_list = list(filter(None, t2_array))
						str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
						str_list.append(time.replace('"',""))
						dictionary_files[source]['file_container']+=','.join(str_list)+'\n'
			t2_file.close()
		else:
			sys.exit("The file %s or directory do not exist"%output_t2_file)

		print(dictionary_files)

		#Creates a file for every GEN elemen
		for source in dictionary_files:
			t2_file_out=open(dictionary_files[source]['filename'], "w")
			t2_file_out.write(dictionary_files[source]['file_container'])
			t2_file_out.close()	

	conn.close()

def gen_csv(input_dictionary):
	"""It generates an output file containing flow and flowing enthalpy for each GEN element from the T2 .csv output file. As a convention the library it is suggested to use GEN for any source/sink that is not a well

	Parameters
	----------
	input_dictionary : dictionary
	 Dictionary contaning the path and name of database on keyword 'db_path'.

	Returns
	-------
	file
	  {GEN}_{BLOCK}_{NICKNAME}.txt : on  ../output/mh/txt

	"""

	db_path=input_dictionary['db_path']

	#List the GEN elements
	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	data_source=pd.read_sql_query("SELECT well,blockcorr,source_nickname FROM t2wellsource WHERE source_nickname  LIKE'GEN*' ORDER BY source_nickname;",conn)

	#Initialize a dictionary containing the file path and name.
	dictionary_files={}
	for n in range(len(data_source)):
		well=data_source['well'][n]
		blockcorr=data_source['blockcorr'][n]
		source=data_source['source_nickname'][n]
		dictionary_files[well]={'filename':"../output/mh/txt/%s_%s_%s_evol_mh.dat"%(well,blockcorr,source),'file_container':"",'blockcorr':blockcorr,'source':source}
		dictionary_files[well]['file_container']+="ELEMENT,SOURCEINDEX,GENERATION RATE,ENTHALPY,FF(GAS),FF(AQ.),P(WB),TIME\n"

	#It reads the gener.csv output file line by line and store the data from each GEN element
	output_t2_file="../model/t2/t2_gener.csv"
	if os.path.isfile(output_t2_file):
		t2_file=open(output_t2_file, "r")
		for t2_line in t2_file:
			if "TIME [sec]" in t2_line:
				try:
					time = t2_line.rstrip().split(" ")[3]
				except IndexError:
					time = t2_line.rstrip().split(" ")[2]
			for well in dictionary_files:
				if dictionary_files[well]['blockcorr'] in t2_line and dictionary_files[well]['source'] in t2_line:
					t2_array=t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list = [val.replace(',',"") for val in str_list ]
					str_list = [str_list[1].replace('"',""), str_list[2].replace('"',""), str_list[6], str_list[7], str_list[8], str_list[9], str_list[10]]
					str_list.append(time.replace('"',""))
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
		t2_file.close()
	else:
		sys.exit("The file %s or directory do not exist"%output_t2_file)

	#Creates a file for every GEN elemen
	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()


def eleme_CSV(input_dictionary, path = None):

	db_path=input_dictionary['db_path']

	#List the GEN elements
	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	dictionary_files = {}
	data_block=pd.read_sql_query("SELECT well, blockcorr FROM t2wellblock  ORDER BY blockcorr;",conn)
	if len(data_block)>0:
		for n, blockcorr in enumerate(data_block['blockcorr']):
			dictionary_files[data_block['well'][n]]={'filename':"../output/PT/evol/%s_PT_evol.dat"%(data_block['well'][n]),\
			                                                'file_container':"",'blockcorr':blockcorr}
	else:
		sys.exit("Store data on the table t2wellblock")

	#It reads the gener.csv output file line by line and store the data from each GEN element
	poss_names = ["../model/t2/t2_XYZ.csv", "../model/t2/t2_eleme.csv"]
	poss_names.append(path)
	output_t2_file = None 
	for file in poss_names:
		if os.path.isfile(file):
			output_t2_file = file

	if output_t2_file == None :
		sys.exit("No output file located")
	else:
		t2_file=open(output_t2_file, "r")
		for i, t2_line in enumerate(t2_file):
			if i == 0:
				t2_array=t2_line.rstrip().split(" ")
				str_list = list(filter(None, t2_array))
				str_list = [val  for val in str_list if '(' not in val or ')' not in val]
				str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
				str_list.append("TIME")
				for well in dictionary_files:
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
			if "TIME [sec]" in t2_line:
				try:
					time = t2_line.rstrip().split(" ")[3]
				except IndexError:
					time = t2_line.rstrip().split(" ")[2]
			for well in dictionary_files:
				if dictionary_files[well]['blockcorr'] in t2_line:
					t2_array=t2_line.rstrip().split(" ")
					str_list = list(filter(None, t2_array))
					str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
					str_list.append(time.replace('"',""))
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
		t2_file.close()

	#Creates a file for every block elemen
	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()


def eleme_CSV_PT(input_dictionary, path = None, def_time = 300):

	db_path=input_dictionary['db_path']

	#List the GEN elements
	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	dictionary_files = {}
	data_block=pd.read_sql_query("SELECT well, blockcorr FROM t2wellblock  ORDER BY blockcorr;",conn)
	if len(data_block)>0:
		for n, blockcorr in enumerate(data_block['blockcorr']):
			dictionary_files[data_block['well'][n]]={'filename':"../output/PT/txt/%s_PT.dat"%(data_block['well'][n]),\
			                                                'file_container':"",'blockcorr':blockcorr}
	else:
		sys.exit("Store data on the table t2wellblock")

	time = -1E50

	#It reads the gener.csv output file line by line and store the data from each GEN element
	poss_names = ["../model/t2/t2_XYZ.csv", "../model/t2/t2_eleme.csv"]
	poss_names.append(path)
	output_t2_file = None 
	for file in poss_names:
		if os.path.isfile(file):
			output_t2_file = file

	if output_t2_file == None :
		sys.exit("No output file located")
	else:
		t2_file=open(output_t2_file, "r")
		for i, t2_line in enumerate(t2_file):
			if i == 0:
				t2_array=t2_line.rstrip().split(" ")
				str_list = list(filter(None, t2_array))
				str_list = [val  for val in str_list if '(' not in val or ')' not in val]
				str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
				for well in dictionary_files:
					dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
			if "TIME [sec]" in t2_line:
				try:
					time = float(t2_line.rstrip().split(" ")[3].replace('"',""))
				except IndexError:
					time = float(t2_line.rstrip().split(" ")[2].replace('"',""))
			if time == def_time:
				for well in dictionary_files:
					if dictionary_files[well]['blockcorr'] in t2_line:
						t2_array=t2_line.rstrip().split(" ")
						str_list = list(filter(None, t2_array))
						str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
						dictionary_files[well]['file_container']+=','.join(str_list)+'\n'
			if time > def_time:
				break
		t2_file.close()

	#Creates a file for every block elemen
	for well in dictionary_files:
		t2_file_out=open(dictionary_files[well]['filename'], "w")
		t2_file_out.write(dictionary_files[well]['file_container'])
		t2_file_out.close()	

	conn.close()

def t2_CSV_to_json(itime=None, all_times =False, path = None, num = None, last = False):
	"""It creates a severals or a single json file from the output file from the TOUGH2 run

	Parameters
	----------
 	itime: float
	  It defines a time at which a the parameters from the blocks are extracted into json file. Must be on the same units as the TOUGH2 output files (days, seconds, etc.)
	save_full: bool
	  If True it creates a single output json file

	Returns
	-------
	files
	  t2_ouput_{time}.json: on ../output/PT/json/evol/
	file
	  t2_output:  on ../output/PT/json/

	Attention
	---------
	When t2_output is saved, the large could be too large for the system to handle it

	Examples
	--------
	>>> t2_to_json()
	"""

	block_json_file='../mesh/ELEME.json'

	if os.path.isfile(block_json_file):
		with open('../mesh/ELEME.json') as file:
		  	blocks=json.load(file)
	else:
		sys.exit("The file %s or directory do not exist, run ELEM_to_json from regeo_mesh"%output_t2_file)		  	


	#It reads the gener.csv output file line by line and store the data from each GEN element
	poss_names = ["../model/t2/t2_XYZ.csv", "../model/t2/t2_eleme.csv"]
	poss_names.append(path)
	output_t2_file = None 
	for file in poss_names:
		if os.path.isfile(file):
			output_t2_file = file

	if output_t2_file == None :
		sys.exit("No output file located")
	else:
		
		cnt_t = 0
		"""
		file =  open(output_t2_file, 'r')
		data_file = file.read()
		count = data_file.count("TIME")
		file.close()
		"""

		count = 400

		if num != None:
			positions = np.linspace(0, count, num).astype(int)
		else:
			positions = range(count)

		t2_file=open(output_t2_file, "r")
		for i, t2_line in enumerate(t2_file):

			if i == 0:
				t2_array=t2_line.rstrip().split(" ")
				param_list = list(filter(None, t2_array))
				param_list = [val  for val in param_list if '(' not in val or ')' not in val]
				param_list = [val.replace(',',"").replace('"','') for val in param_list[1:] ]

			if "TIME [sec]" in t2_line:
				if i >10 and cnt_t in positions: #takes the previous time and save it
					t2_pd=pd.DataFrame.from_dict(data_dictionary,orient='index')
					print(time)
					t2_pd.to_json('../output/PT/json/evol/t2_output_%.0f.json'%float(time),orient="index",indent=2)
				
				try:
					time = float(t2_line.rstrip().split(" ")[3].replace('"',""))
					cnt_t += 1
				except IndexError:
					time = float(t2_line.rstrip().split(" ")[2].replace('"',""))
					cnt_t += 1

				data_dictionary= {}	
				data_dictionary[time] = {}

				if not all_times:
					if time > itime:
						break

			if i != 0 and 'TIME' not in t2_line and cnt_t in positions:
				t2_array=t2_line.rstrip().split(" ")
				str_list = list(filter(None, t2_array))
				block = str_list[1].replace('"','').replace(',',"")
				data_dictionary[time][block]={}
				str_list = [val.replace(',',"").replace('"','') for val in str_list[1:] ]
				for n, param in enumerate(param_list):
					try:
						data_dictionary[time][block][param] = float(str_list[n])
					except ValueError:
						data_dictionary[time][block][param] = str_list[n]


def total_enthalpy(input_dictionary, wells, years = 35):


	#data['date_time'] = pd.date_range(start= input_dictionary['ref_date'], end= input_dictionary['ref_date']+datetime.timedelta(days=years*365.25), freq = 'D' )

	data_p = pd.DataFrame(columns=['date_time', 'mh', 'm', 'h'])
	

	for well in wells:

		data = pd.DataFrame(columns=['date_time', 'mh', 'm', 'h'])
		
		data_x = pd.read_csv("../input/mh/%s_mh.dat"%well, usecols = ['date_time', 'total_flow', 'enthalpy', 'status', 'steam'])
		data_x = data_x.loc[data_x['steam'] > 0]
		data_x['date_time'] = pd.to_datetime(data_x['date_time'] , format="%Y-%m-%d_%H:%M:%S")

		
		data_ix = data_x.copy()
		data_ix.index = data_ix['date_time']
		del data_ix['date_time']
		data_ii_inter_x = data_ix.resample('D').mean().ffill()
		data_ii_inter_x['date_time'] = data_ii_inter_x.index


		if wells[well]['type'] == 'producer':
			data['mh'] = data_ii_inter_x['total_flow']*data_ii_inter_x['enthalpy']
			data['m'] = data_ii_inter_x['total_flow']
			data['date_time'] = data_ii_inter_x['date_time']

			data_p = data_p.append(data, ignore_index = True)

	data_p = data_p.groupby('date_time').sum()

	data_p['h'] = data_p['mh']/data_p['m']

	data_p.to_csv('../input/mh/total_prod_mh.csv' , index = True)



def power(input_dictionary, WHP):

	def power_i(Px,hx,mx,ESC):
		if mx <= 0:
			try:
				quality = IAPWS97(P=Px+0.092,h=hx).x
				power = -quality*mx/ESC
			except NotImplementedError:
				print("Out of bounds, Pressure:", Px,"[bar], Enthalpy: ",hx, "[kJ/kg]")
				power = 0
		else:
			power = 0
		return power
		
	data_12 = pd.DataFrame(columns = ['date_time', 'h', 'm', 'power'])
	data_3 = pd.DataFrame(columns = ['date_time', 'h', 'm', 'power'])

	conn=sqlite3.connect(input_dictionary['db_path'])
	c=conn.cursor()

	well_sources = pd.read_sql_query("SELECT * FROM t2wellsource WHERE well LIKE 'TR-%'",conn)

	for index, row in well_sources.iterrows(): 
		if str(row['well']) in [*WHP]:

			evol_file = '../output/PT/evol/%s_PT_evol.dat'%row['well']

			data_i = pd.read_csv(evol_file)

			temp_i = data_i.loc[ (data_i['ELEM'] == row ['blockcorr']) & (data_i['TIME'] >=0 ), ['PRES_VAP','TIME']]
			temp_i.reset_index(inplace = True)

			file = "../output/mh/txt/%s_%s_%s_evol_mh.dat"%(row['well'],row['blockcorr'],row['source_nickname'])

			data_mh = pd.read_csv(file)

			times = data_mh['TIME']

			dates=[]
			enthalpy = []
			flow_rate = []
			pressures = []
			power = []

			for n in range(len(times)):
				if float(times[n]) >= 0:
					try:
						pressures.append(temp_i['PRES_VAP'][n])
						enthalpy.append(data_mh['ENTH'][n]/1E3)
						flow_rate.append(data_mh['GEN'][n])
						dates.append(input_dictionary['ref_date']+datetime.timedelta(seconds=int(times[n])))
						
						if str(row['well']) in [*WHP]:
							power.append(power_i(WHP[row['well']][0]/1E6,data_mh['ENTH'][n]/1E3,data_mh['GEN'][n],WHP[row['well']][1]))
						else:
							power.append(np.nan)

					except (OverflowError, KeyError):
						print(times[n],"plus",str(times[n]),"wont be plot")

			input_row = {'date_time': dates,
			                 'h': enthalpy,
			                 'm': flow_rate,
			               'BWP': pressures,
			               'power':power}

			output = pd.DataFrame.from_dict(input_row)

			if WHP[row['well']][2] == '1_2':
				data_12 = data_12.append(output, ignore_index = True)
			elif WHP[row['well']][2] == '3':
				data_3 = data_3.append(output, ignore_index = True)

	data_12 = data_12.groupby(['date_time']).sum() 
	data_3 = data_3.groupby(['date_time']).sum() 


	data_12.to_csv('../output/unit_12_power.csv')
	data_3.to_csv('../output/unit_3_power.csv')
		


def field_areas_enthalpy(wells):


	data_p_12 = pd.DataFrame(columns=['date_time', 'mh', 'm', 'h'])
	data_p_3_17 = pd.DataFrame(columns=['date_time', 'mh', 'm', 'h'])
	data_p_3_18 = pd.DataFrame(columns=['date_time', 'mh', 'm', 'h'])
	
	for well in wells:

		data = pd.DataFrame(columns=['date_time', 'mh', 'm', 'h'])
		
		data_x = pd.read_csv("../input/mh/%s_mh.dat"%well, usecols = ['date_time', 'total_flow', 'enthalpy', 'status', 'steam'])
		data_x = data_x.loc[data_x['steam'] > 0]
		data_x['date_time'] = pd.to_datetime(data_x['date_time'] , format="%Y-%m-%d_%H:%M:%S")

		
		data_ix = data_x.copy()
		data_ix.index = data_ix['date_time']
		del data_ix['date_time']
		data_ii_inter_x = data_ix.resample('D').mean().ffill()
		data_ii_inter_x['date_time'] = data_ii_inter_x.index


		data['mh'] = data_ii_inter_x['total_flow']*data_ii_inter_x['enthalpy']
		data['m'] = data_ii_inter_x['total_flow']
		data['date_time'] = data_ii_inter_x['date_time']

		if wells[well][2] == '1_2':
			data_p_12 = data_p_12.append(data, ignore_index = True)
		elif wells[well][2] == '3':
			if '17' in well:
				data_p_3_17 = data_p_3_17.append(data, ignore_index = True)
			elif '18' in well:
				data_p_3_18 = data_p_3_18.append(data, ignore_index = True)

	data_p_12 = data_p_12.groupby('date_time').sum()

	data_p_12['h'] = data_p_12['mh']/data_p_12['m']

	data_p_12.to_csv('../input/mh/unit12_mh.csv' , index = True)


	data_p_3_17 = data_p_3_17.groupby('date_time').sum()

	data_p_3_17['h'] = data_p_3_17['mh']/data_p_3_17['m']

	data_p_3_17.to_csv('../input/mh/unit3_17_mh.csv' , index = True)


	data_p_3_18 = data_p_3_18.groupby('date_time').sum()

	data_p_3_18['h'] = data_p_3_18['mh']/data_p_3_18['m']

	data_p_3_18.to_csv('../input/mh/unit3_18_mh.csv' , index = True)


from T2GEORES import geometry as geomtr
import sqlite3
import os
import pandas as pd
import json

def checktable(table_name,c):
	"""It verifies the existance of a table on the sqlite database

	Parameters
	----------
	table_name : str
	  Table name
	c : cursor
	  Conection to the database

	Returns
	-------
	int
		check: if table exists returns 1

	Examples
	--------
	>>>  checktable(table_name,c)
	"""

	query="SELECT COUNT(name) from sqlite_master WHERE type='table' AND name='%s'"%(table_name)
	c.execute(query)
	if c.fetchone()[0]==1:
		check=1
	else:
		check=0
	return check

def db_creation(input_dictionary):
	"""It creates a sqlite databa base 

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary contaning the path and name of database on keyword 'db_path', usually on '../input/'

	Returns
	-------
	database
		name: database on desire path

	Note
	----
	The tables: wells, survey, PT, mh, drawdown, cooling, wellfeedzone, t2wellblock, t2wellsource, layers and t2PTout are generated

	Examples
	--------
	>>> db_creation(input_dictionary)
	"""

	db_path=input_dictionary['db_path']

	if not os.path.isfile(db_path):
		conn=sqlite3.connect(db_path)
		c = conn.cursor()

		# Create table - wells
		if checktable('wells',c)==0:
			c.execute('''CREATE TABLE wells
		             ([well] TEXT PRIMARY KEY,
		              [type] TEXT,
		              [east] REAL,
		              [north] REAL,
		              [elevation] REAL,
		              [lnr_init] REAL,
		              [lnr_end] REAL,
		              [lnr_D] TEXT,
		              [ptube_init] REAL,
		              [ptube_end] REAL,
		              [ptube_D] TEXT,
		              [drilldate] datetime)''')

		#Create table - survey
		if checktable('survey',c)==0:
			c.execute('''CREATE TABLE  survey
						 ([well] TEXT,
						 [MeasuredDepth] REAL,
						 [Delta_east] REAL,
						 [Delta_north] REAL)''')

		#Create table - PT
		if checktable('PT',c)==0:
			c.execute('''CREATE TABLE  PT
						 ([well] TEXT,
						 [MeasuredDepth] REAL,
						 [Pressure] REAL,
						 [Temperature] REAL)''')

		#Create table - raw mh
		if checktable('mh',c)==0:
			c.execute('''CREATE TABLE  mh
						 ([well] TEXT,
						 [type] TEXT,
						 [date_time] datetime,
						 [steam_flow] REAL,
						 [liquid_flow] REAL,
						 [flowing_enthalpy] REAL,
						 [well_head_pressure] REAL)''')

		#Create table - filtered mh
		if checktable('mh_f',c)==0:
			c.execute('''CREATE TABLE  mh_f
						 ([well] TEXT,
						 [status] TEXT,
						 [date_time] datetime,
						 [steam_flow] REAL,
						 [liquid_flow] REAL,
						 [flowing_enthalpy] REAL,
						 [well_head_pressure] REAL)''')

		#Create table - drawdown
		if checktable('drawdown',c)==0:
			c.execute('''CREATE TABLE  drawdown
						 ([well] TEXT,
						 [date_time] datetime,
						 [TVD] REAL,
						 [pressure] REAL)''')

		#Create table - cooling
		if checktable('cooling',c)==0:
			c.execute('''CREATE TABLE  cooling
						 ([well] TEXT,
						 [date_time] datetime,
						 [TVD] REAL,
						 [temp] REAL)''')

		#Create table - wellfeedzone
		if checktable('wellfeedzone',c)==0:
			c.execute('''CREATE TABLE  wellfeedzone
						 ([well] TEXT,
						 [MeasuredDepth] REAL,
						 [contribution] REAL)''')

		#Create table - TOUGH2 well block(correlative)
		if checktable('t2wellblock',c)==0:
			c.execute('''CREATE TABLE  t2wellblock
						 ([well] TEXT PRIMARY KEY,
						 [blockcorr] TEXT)''')

		#Create table - TOUGH2 well source
		if checktable('t2wellsource',c)==0:
			c.execute('''CREATE TABLE  t2wellsource
						 ([well] TEXT,
						 [blockcorr] TEXT ,
						 [rocktype] TEXT ,
						 [flow_type] TEXT ,
						 [source_nickname] TEXT PRIMARY KEY)''')

		#Create table - layers levels
		if checktable('layers',c)==0:
			c.execute('''CREATE TABLE  layers
						 ([correlative] TEXT PRIMARY KEY,
						 [top] REAL,
						 [middle] REAL,
						 [bottom] REAL)''')
			
		#Create table - stores ELEME section of mesh
		if checktable('ELEME',c)==0:
			c.execute('''CREATE TABLE  ELEME
						 ([model_version] REAL,
						 [model_output_timestamp] timestamp,
						 [ELEME] TEXT,
						 [NSEQ] REAL,
						 [NADD] REAL,
						 [MA1] REAL,
						 [MA2] REAL,
						 [VOLX] REAL,
						 [AHTX] REAL,
						 [PMX] REAL,
						 [X] REAL,
						 [Y] REAL,
						 [Z] REAL,
						 [LAYER_N] REAL,
						 [h] REAL)''')

		#Create table - stores CONNE section of mesh
		if checktable('CONNE',c)==0:
			c.execute('''CREATE TABLE  CONNE
						 ([model_version] REAL,
						 [model_output_timestamp] timestamp,
						 [ELEME1] TEXT,
						 [ELEME2] TEXT,
						 [NSEQ] REAL,
						 [NAD1] REAL,
						 [NAD2] REAL,
						 [ISOT] REAL,
						 [D1] REAL,
						 [D2] REAL,
						 [AREAX] REAL,
						 [BETAX] REAL,
						 [SIGX] REAL)''')

		#Create table - stores segment
		if checktable('segment',c)==0:
			c.execute('''CREATE TABLE  segment
						 ([model_version] REAL,
						 [model_output_timestamp] timestamp,
						 [x1] REAL,
						 [y1] REAL,
						 [x2] REAL,
						 [y2] REAL,
						 [redundant] REAL,
						 [ELEME1] TEXT,
						 [ELEME2] TEXT)''')

		#Create table - PT out
		if checktable('t2PTout',c)==0:
			c.execute('''CREATE TABLE  t2PTout
						 ([blockcorr] TEXT PRIMARY KEY,
						 [x] REAL,
						 [y] REAL,
						 [z] REAL,
						 [index] REAL,
						 [P] REAL,
						 [T] REAL,
						 [SG] REAL,
						 [SW] REAL,
						 [X1] REAL,
						 [X2] REAL,
						 [PCAP] REAL,
						 [DG] REAL,
						 [DW] REAL)''')

		#Create table - stores flows TOUGH2 output section
		if checktable('t2FLOWSout',c)==0:
			c.execute('''CREATE TABLE  t2FLOWSout
						 ([model_version] REAL,
						 [model_output_timestamp] timestamp,
						 [ELEME1] TEXT,
						 [ELEME2] TEXT,
						 [INDEX] INT,
						 [FHEAT] REAL,
						 [FLOH] REAL,
						 [FLOF] REAL,
						 [FLOG] REAL,
						 [FLOAQ] REAL,
						 [FLOWTR2] REAL,
						 [VELG] REAL,
						 [VELAQ] REAL,
						 [TURB_COEFF] REAL,
						 [model_time] REAL)''')

		#Create table - stores flows directions from every block
		if checktable('t2FLOWVectors',c)==0:
			c.execute('''CREATE TABLE  t2FLOWVectors
						 ([model_version] REAL,
						 [model_output_timestamp] timestamp,
						 [ELEME] TEXT,
						 [FHEAT_x] REAL,
						 [FHEAT_y] REAL,
						 [FHEAT_z] REAL,
						 [FLOH_x] REAL,
						 [FLOH_y] REAL,
						 [FLOH_z] REAL,
						 [FLOF_x] REAL,
						 [FLOF_y] REAL,
						 [FLOF_z] REAL,
						 [FLOG_x] REAL,
						 [FLOG_y] REAL,
						 [FLOG_z] REAL,
						 [FLOAQ_x] REAL,
						 [FLOAQ_y] REAL,
						 [FLOAQ_z] REAL,
						 [FLOWTR2_x] REAL,
						 [FLOWTR2_y] REAL,
						 [FLOWTR2_z] REAL,
						 [VELG_x] REAL,
						 [VELG_y] REAL,
						 [VELG_z] REAL,
						 [VELAQ_x] REAL,
						 [VELAQ_y] REAL,
						 [VELAQ_z] REAL,
						 [TURB_COEFF_x] REAL,
						 [TURB_COEFF_y] REAL,
						 [TURB_COEFF_z] REAL,
						 [model_time] REAL)''')

		conn.commit()
		conn.close()

def insert_wells_sqlite(input_dictionary):
	"""It stores the data contain on the ubication.csv file and stores it on the database

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	The well name is written as primary key. Thus, if the coordinates of the file ubication.csv change, it is better to 
	eliminate the records and rerun this function again. Some print are expected.

	Examples
	--------
	>>>  insert_wells_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c = conn.cursor()

	wells=pd.read_csv(source_txt+'ubication.csv')

	wells['drilldate'] = pd.to_datetime(wells['drilldate'],format="%Y%m%d")

	for  index,row in wells.iterrows():
		try:
			q="INSERT INTO wells(well,type,east,north,elevation,drilldate) VALUES ('%s','%s',%s,%s,%s,'%s')"%\
			(row['well'],row['type'],row['east'],row['north'],row['masl'],row['drilldate'])
			c.execute(q)
			conn.commit()
		except sqlite3.IntegrityError:
			print("The well %s is already on the database")
	conn.close()

def insert_feedzone_to_sqlite(input_dictionary):
	"""It stores the data contain on the ubication.csv file and stores it on the database

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Examples
	--------
	>>>  insert_feedzone_to_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	feedzones=pd.read_csv(source_txt+'well_feedzone.csv',delimiter=',')


	for index,row in feedzones.iterrows():
		q="INSERT INTO wellfeedzone(well,MeasuredDepth,contribution) VALUES ('%s',%s,%s)"%\
		(row['well'],row['MD'],row['contribution'])
		c.execute(q)
		conn.commit()
	conn.close()

def insert_survey_to_sqlite(input_dictionary):
	"""It stores all the data contain on the subfolder survey from the input file folder.

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	The survey for every well must have the next headers MeasuredDepth,Delta_north,Delta_east

	Examples
	--------
	>>>  insert_survey_to_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	
	for f in os.listdir(source_txt+'survey/'):
		if os.path.isfile(os.path.join(source_txt, 'survey/',f)):
			well_name=f.replace("'","").replace("_MD.dat","")
			well_file=os.path.join(source_txt, 'survey/',f)
			survey=pd.read_csv(well_file)
			for index, row in survey.iterrows():
				q="INSERT INTO survey(well,MeasuredDepth,Delta_north,Delta_east) VALUES ('%s',%s,%s,%s)"%\
				(well_name,row['MeasuredDepth'],row['Delta_north'],row['Delta_east'])
				c.execute(q)
				conn.commit()
	conn.close()

def insert_PT_to_sqlite(input_dictionary):
	"""It stores all the data contain on the subfolder PT from the input file folder.

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	The PT for every well must have the next headers MD,P,T. The file name must be well_MDPT.dat

	Examples
	--------
	>>>  insert_PT_to_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'PT'):
		if os.path.isfile(source_txt+'PT/'+f):
			if '_MDPT' in f:
				well_name=f.replace("'","").replace("_MDPT.dat","")
				if os.path.isfile(source_txt+'PT/'+f):
					PT=pd.read_csv(source_txt+'PT/'+f)
					for index, row in PT.iterrows():
						q="INSERT INTO PT(well,MeasuredDepth,Pressure,Temperature) VALUES ('%s',%s,%s,%s)"%\
						(well_name,row['MD'],row['P'],row['T'])
						c.execute(q)
						conn.commit()
	conn.close()

def insert_drawdown_to_sqlite(input_dictionary):
	"""It stores all the data contain on the subfolder drawdown from the input file folder.

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	The drawdown register on every well must have the next headers datetime,TVD,pressure. The file name must be well_DD.dat

	Examples
	--------
	>>>  insert_drawdown_to_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'drawdown'):
		well_name=f.replace("'","").replace("_DD.dat","")
		if os.path.isfile(source_txt+'drawdown/'+f) and f!='p_res.csv':
			DD=pd.read_csv(source_txt+'drawdown/'+f)
			DD['well'] = well_name
			DD.rename(columns={'datetime': 'date_time', 'temperature': 'temp'}, inplace=True)
			DD.to_sql('drawdown',if_exists='append',con=conn,index=False)
			
	conn.close()

def insert_cooling_to_sqlite(input_dictionary):
	"""It stores all the data contain on the subfolder cooling from the input file folder.

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	The cooling register on every well must have the next headers datetime,TVD,temperature. The file name must be well_C.dat

	Examples
	--------
	>>>  insert_cooling_to_sqlite(input_dictionary)
	"""


	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'cooling'):
		well_name=f.replace("'","").replace("_C.dat","")
		if os.path.isfile(source_txt+'cooling/'+f):
			CC=pd.read_csv(source_txt+'cooling/'+f)
			CC['well'] = well_name
			CC.rename(columns={'datetime': 'date_time', 'temperature': 'temp'}, inplace=True)
			CC.to_sql('cooling',if_exists='append',con=conn,index=False)
			
	conn.close()

def insert_raw_mh_to_sqlite(input_dictionary):
	"""It stores all the data contain on the subfolder mh from the input file folder.

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	Every file contains information about the flow rate and flowing enthalpy of the wells. Every register must contain the next headers:
	type,date-time,steam,liquid,enthalpy,WHPabs. The file name must be name well_mh.dat

	Examples
	--------
	>>>  insert_mh_to_sqlite(input_dictionary)
	"""

    db_path=input_dictionary['db_path']
    source_txt=input_dictionary['source_txt']
    
    conn=sqlite3.connect(db_path)
    c=conn.cursor()
    for f in os.listdir(source_txt+'mh'):
        print(f)
        well_name=f.replace("'","").replace("_mh.dat","")
        if os.path.isfile(source_txt+'mh/'+f):
            mh=pd.read_csv(source_txt+'mh/'+f)
            mh['well'] = well_name
            mh.rename(columns={'steam': 'steam_flow', 'liquid': 'liquid_flow', 'enthalpy': 'flowing_enthalpy',  'WHPabs': 'well_head_pressure', 'status': 'type'}, inplace=True)
            mh.to_sql('mh',if_exists='append',con=conn,index=False)
    conn.close()

def insert_filtered_mh_to_sqlite(input_dictionary, single_well = None):
	"""It stores all the data contain on the subfolder mh from the input file folder.

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	Every file contains information about the flow rate and flowing enthalpy of the wells. Every register must contain the next headers:
	type,date-time,steam,liquid,enthalpy,WHPabs. The file name must be name well_mh.dat

	Examples
	--------
	>>>  insert_mh_to_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	if single_well == None:
		for f in os.listdir(source_txt+'mh/filtered'):
			print(f)
			well_name=f.replace("'","").replace("_week_avg.csv","")
			if os.path.isfile(source_txt+'mh/filtered/'+f) and well_name in input_dictionary['WELLS']:
				mh=pd.read_csv(source_txt+'mh/filtered/'+f)
				mh['well'] = well_name
				mh.rename(columns={'steam': 'steam_flow', 'liquid': 'liquid_flow', 'enthalpy': 'flowing_enthalpy',  'WHPabs': 'well_head_pressure'}, inplace=True)
				mh.to_sql('mh_f',if_exists='append',con=conn,index=False)
	else:
		file_name = single_well + "_week_avg.csv"
		if os.path.isfile(source_txt+'mh/filtered/'+file_name) and single_well in input_dictionary['WELLS']:
			mh=pd.read_csv(source_txt+'mh/filtered/'+file_name)
			mh['well'] = single_well
			mh.rename(columns={'steam': 'steam_flow', 'liquid': 'liquid_flow', 'enthalpy': 'flowing_enthalpy',  'WHPabs': 'well_head_pressure'}, inplace=True)
			mh.to_sql('mh_f',if_exists='append',con=conn,index=False)
			
	conn.close()

def replace_mh(wells_to_replace,input_dictionary):
	"""It stores all the data contain on the subfolder mh from the input file folder from some selected wells

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file
	wells_to_replace: list
	  Contains the data from the wells which flow data will be replace

	Note
	----
	Every file contains information about the flow rate and flowing enthalpy of the wells. Every register must contain the next headers:
	type,date-time,steam,liquid,enthalpy,WHPabs. The file name must be name well_mh.dat

	Examples
	--------
	>>>  replace_mh(wells_to_replace=['WELL-1','WELL-2'],input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	for f in os.listdir(source_txt+'mh'):
		well_name=f.replace("'","").replace("_mh.dat","")
		if well_name in wells_to_replace:
			q="DELETE FROM mh WHERE well='%s'"%well_name
			c.execute(q)
			conn.commit()
			if os.path.isfile(source_txt+'mh/'+f):
				print(source_txt+'mh/'+f)
				mh=pd.read_csv(source_txt+'mh/'+f)
				for index, row in mh.iterrows():
					q="INSERT INTO mh(well,type,date_time,steam_flow,liquid_flow,flowing_enthalpy,well_head_pressure) VALUES ('%s','%s','%s',%s,%s,%s,%s)"%\
					(well_name,row['type'],row['date-time'],row['steam'],row['liquid'],row['enthalpy'],row['WHPabs'])
					c.execute(q)
					conn.commit()
	conn.close()

def sqlite_to_json(input_dictionary):
	"""It exports the database into a single json file

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	It creates from the tables wells, survey, PT, mh, drawdown, wellfeedzone, t2wellblock and t2wellsource a  json file, using as ID the field well

	Other Parameters
	----------------
	dictionary
		levels : configuration dictionary, indicates the table name, the ID to use and the field to use on each table

	Examples
	--------
	>>>  sqlite_to_json(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	source_txt=input_dictionary['source_txt']

	levels={'wells':['well'],
			'survey':['well'],
			'PT':['well','MeasuredDepth'],
			'mh':['well','date_time'],
			'drawdown':['well','date_time'],
			'wellfeedzone':['well'],
			't2wellblock':['well'],
			't2wellsource':['well']}

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	dict_to_json=dict((n,{}) for n in levels.keys())
	
	for lv in levels:
		cnt=0
		for lv2 in levels[lv]:
			
			if len(levels[lv])==1:

				query="SELECT DISTINCT %s from %s"%(lv2,lv)
				c.execute(query)
				rows=c.fetchall()
				dict_keys=[]
				for row in rows:
					dict_keys.append(str(row[0]))

				dict_of_wells_on_table=dict((n,{}) for n in dict_keys)
				dict_to_json[lv]=dict_of_wells_on_table

				query="PRAGMA table_info(%s)"%lv
				c.execute(query)
				rows=c.fetchall()
				colums=[]
				for row in rows:
					colums.append(str(row[1]))

				partq=[]

				q_string=""
				for nv in colums:
					if nv not in levels[lv] :
						partq.append(nv)
						q_string+=" %s,"%(nv)

				for nv in dict_keys:
					query="SELECT %s from %s WHERE %s='%s'"%(q_string[0:-1],lv,lv2,nv)

					conn=sqlite3.connect(db_path)
					conn.row_factory = sqlite3.Row
					c=conn.cursor()
					rows=c.execute(query).fetchall()

					dict_to_json[lv][nv]=[dict(ix) for ix in rows]

			elif len(levels[lv])>1:
				if cnt==0:
					query="SELECT DISTINCT %s from %s"%(lv2,lv)
					prev_key=lv2

					c.execute(query)
					rows=c.fetchall()
					dict_keys=[]
					for row in rows:
						dict_keys.append(str(row[0]))

					dict_of_wells_on_table=dict((n,{}) for n in dict_keys)
					dict_to_json[lv]=dict_of_wells_on_table
					cnt+=1

				else:

					#Extract the colums which are not on the results
					query="PRAGMA table_info(%s)"%lv
					c.execute(query)
					rows=c.fetchall()
					colums=[]
					for row in rows:
						colums.append(str(row[1]))

					partq=[]

					q_string=""
					for nv in colums:
						if nv not in levels[lv] :
							partq.append(nv)
							q_string+=" %s,"%(nv)

					for prev_result in dict_keys:

						query="SELECT DISTINCT %s from %s WHERE %s='%s'"%(lv2,lv,prev_key,prev_result)

						c.execute(query)
						rows=c.fetchall()
						dict_keys=[]
						for row in rows:
							dict_keys.append(str(row[0]))

						dict_of_wells_on_table=dict((n,{}) for n in dict_keys)
						dict_to_json[lv][prev_result]=dict_of_wells_on_table

						for nv in dict_keys:

							query="SELECT %s from %s WHERE %s='%s' AND %s='%s' "%(q_string[0:-1],lv,lv2,nv,prev_key,prev_result)

							conn=sqlite3.connect(db_path)
							conn.row_factory = sqlite3.Row
							c=conn.cursor()
							rows=c.execute(query).fetchall()

							dict_to_json[lv][prev_result][nv]=[dict(ix) for ix in rows]
	
	with open(source_txt+'model.json', 'w') as jlines:
		json.dump(dict_to_json, jlines,indent=4,sort_keys=True)
	

	conn.close()

def insert_wellblock_to_sqlite(input_dictionary):
	"""It stores the file wells_correlative.txt coming from regeo_mesh on the table t2wellblock

	Parameters
	----------
	input_dictionary : dictionary
	  A dictionary containing the  name and path of database ans well types WELLS, MAKE_UP_WELLS and NOT_PRODUCING_WELLS

	Note
	----
	The correlative from each wells just contains four characthers. Some print output are expected.


	Examples
	--------
	>>>  insert_wellblock_to_sqlite(input_dictionary)
	"""

	db_path=input_dictionary['db_path']

	if os.path.isfile('../mesh/wells_correlative.txt'):

		with open('../mesh/wells_correlative.txt',encoding='utf8') as json_file:
		    wells_correlative=json.load(json_file)


		conn=sqlite3.connect(db_path)
		c=conn.cursor()

		types=['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']
		wells=[]
		for scheme in types:
			try:
				for well in input_dictionary[scheme]:
					wells.append(well)
			except KeyError:
					pass
		print(wells_correlative['P-1'])
		for name in wells:
			try:
				blocknumer=wells_correlative[name][0]
				q="INSERT INTO t2wellblock(well,blockcorr) VALUES ('%s','%s')"%(name,blocknumer)
				c.execute(q)
				conn.commit()
			except KeyError:
				print("Error")
				pass
			except sqlite3.IntegrityError:
				print("The block  number is already assing for the well: %s"%name)
		conn.close()

	else:
		print("The file well_correlative.txt do not exists")

def insert_layers_to_sqlite(input_dictionary):
	"""It stores layers and elevations

	Parameters
	----------
	input_dictionary: dictionary
	  Dictionary containing the path and name of database and the path of the input file

	Note
	----
	The data information comes from the input_dictionary which must have a keyword layer with a similar structure as following: 'LAYERS':{1:['A',100],2:['B', 100]}. Some print output are expected.

	Examples
	--------
	>>>  insert_layers_to_sqlite(input_dictionary)
	"""

	layers_info=geomtr.vertical_layers(input_dictionary)
	top=layers_info['top']
	correlative=layers_info['name']
	middle=layers_info['middle']
	bottom=layers_info['bottom']

	conn=sqlite3.connect(input_dictionary['db_path'])
	c=conn.cursor()

	for n in range(len(top)):
		q="INSERT INTO layers(correlative,top,middle,bottom) VALUES ('%s',%s,%s,%s)"\
			%(correlative[n],top[n],middle[n],bottom[n])
		try:
			c.execute(q)
			conn.commit()
		except sqlite3.IntegrityError:
			print("The layer %s is already define, correlative is a PRIMARY KEY"%correlative[n])
	conn.close()


#Not documented

def src_rocktype(input_dictionary):

	elem_file='../mesh/ELEME.json'

	if os.path.isfile(elem_file):
		with open(elem_file) as file:
		  	blocks=json.load(file)


	db_path=input_dictionary['db_path']

	#List the GEN elements
	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	data_source=pd.read_sql_query("SELECT well, blockcorr,source_nickname FROM t2wellsource ORDER BY source_nickname;",conn)

	
	data_source['rocktype'] = None

	for n in range(len(data_source)):
		blockcorr=data_source['blockcorr'][n]
		rocktype_n = blocks[blockcorr]['MA1']
		data_source['rocktype'][n] = rocktype_n

	data_source.to_sql('t2wellsource',if_exists='replace',con=conn,index=False, index_label = 'source_nickname')

	conn.close()



from distutils.core import setup

with open("README.md","r",encoding='utf-8') as fh:
    long_description=fh.read()

setup(
    name='T2GEORES',
    version='0.1.96',
    packages=['T2GEORES',],
    license='MIT',
    author='Erick Jiménez',
    author_email="erick@grogtp.is",
    url ="https://github.com/jejimenezm/T2GEORES",
    download_url='https://github.com/jejimenezm/T2GEORES/releases/download/T2GEORES-0.1.96/T2GEORES-0.1.96.tar.gz',
    long_description_content_type='text/markdown',
    long_description=long_description,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    install_requires=[
        'pandas>=1.1.3',
        'numpy>=1.14.5',
        'matplotlib>=3.3.2',
        'iapws>=1.5.2',
		'scipy>=1.5.3',
        'pyshp>=2.2.0',
        'pyvista>=0.33.3',
        'geopandas>=0.10.2'
    ],
)


from T2GEORES import formats as formats
import numpy as np
import csv
from scipy import interpolate
import pandas as pd
import sys
import shapefile
import json
from iapws import IAPWS97
import sqlite3
import math


def vertical_layers(input_dictionary):
	"""It returns the layers information on a dictionary

	Returns
	-------
	input_dictionary : dictionary
	  Contains the infomation of the layer under the keyword 'LAYER' and 'z_ref'.

	Returns
	-------
	dictionary
		layers_info: dictionary with keywords; name, top, middle and bottom. The last three are float values
	
	"""

	layers_info={'name':[],'top':[],'middle':[],'bottom':[]}

	depth=0

	layers=input_dictionary['LAYERS']
	for cnt, layer in enumerate(sorted(layers)):
		depth+=layers[layer][1]
		z_bottom=input_dictionary['z_ref']-depth
		if cnt==0:
			z_top=input_dictionary['z_ref']
		else:
			z_top+=-layers[last_key][1]
		last_key=layer

		z_mid=(z_top+z_bottom)*0.5

		layers_info['name'].append(layers[layer][0])
		layers_info['top'].append(z_top)
		layers_info['middle'].append(z_mid)
		layers_info['bottom'].append(z_bottom)

	return layers_info

def MD_to_TVD(well,depth):
	"""It returns the coordinates X,Y and Z at a desired depth.

	Parameters
	----------
	well : str
	  Selected well
	depth : float
	  Measure depth at which the X,Y,Z coordinate is needed

	Returns
	-------
	float
	  x_V :  x position of desire point
	float
	  y_V :  y position of desire point
	float
	  z_V :  z position of desire point
	  
	Attention
	---------
	The input information comes from the files input/ubication.csv and  input/survey/{well}_MD.dat.

	Note
	----
	A linear regression is used.

	Examples
	--------
	>>> MD_to_TVD('WELL-1',500)
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r')) #'rb'
	dict_ubication={}
	for line in reader:
		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		x[j]=x_0+DeltaX[j]
		y[j]=y_0+DeltaY[j]
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funxmd=interpolate.interp1d(MD,x)
	funymd=interpolate.interp1d(MD,y)
	funzmd=interpolate.interp1d(MD,z)

	try:
		x_out=funxmd(depth)
		y_out=funymd(depth)
		z_out=funzmd(depth)
	except ValueError:
		x_out=np.nan
		y_out=np.nan
		z_out=np.nan
	return x_out,y_out,z_out

def MD_to_TVD_one_var_array(well,var_array,MD_array):
	"""It returns the position X, Y and Z for every point on a log along Pressure or Temperature.

	Parameters
	----------
	well : str
	  Selected well
	var_array : array
	  It contains the log of pressure or temperature
	MD_array :array
	  It contains the measure depth values corresponding with every point on var_array

	Returns
	-------
	float
	  x_V :  x position of desire point
	float
	  y_V :  y position of desire point
	float
	  z_V :  z position of desire point
	array
	  v_V : contains the values of pressure or temperature for every coordinate
	  
	Attention
	---------
	The input information comes from the files input/ubication.csv and  input/survey/{well}_MD.dat.

	Note
	----
	A linear regression is used.

	Examples
	--------
	>>> MD_to_TVD_one_var_array('WELL-1',data['Temp'],data['MD'],100)
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r'))
	dict_ubication={}
	for line in reader:
		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		x[j]=x_0+DeltaX[j]
		y[j]=y_0+DeltaY[j]
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funxmd=interpolate.interp1d(MD,x)
	funymd=interpolate.interp1d(MD,y)
	funzmd=interpolate.interp1d(MD,z)

	#Working with variable

	funcV=interpolate.interp1d(MD_array,var_array)
	x_V=[]
	y_V=[]
	z_V=[]
	var_V=[]

	#MD_array_reconvert=np.linspace(min(MD_array),max(MD_array),num_points)

	for value in MD_array:
		try:
			x_V.append(funxmd(value))
			y_V.append(funymd(value))
			z_V.append(funzmd(value))
			var_V.append(funcV(value))
		except ValueError:
			pass

	return x_V,y_V,z_V,var_V

def TVD_to_MD(well,TVD):
	"""It returns the measure depth position for a well based on a true vertical depth

	Parameters
	----------
	well : str
	  Selected well
	TVD : float
	  Desire true vertical depth

	Returns
	-------
	float
	  MD :  measure depth

	Attention
	---------
	The input information comes from the files input/ubication.csv and  input/survey/{well}_MD.dat.

	Note
	----
	A linear regression is used.

	Examples
	--------
	>>> TVD_to_MD('WELL-1',-100)
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r')) #'rb'
	dict_ubication={}
	for line in reader:

		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funzmd=interpolate.interp1d(z,MD)


	try:
		MD=funzmd(TVD)
	except ValueError:
		MD=np.nan
	return MD

def TVD_to_MD_array(well,TVD_array):
	"""It returns the measure depths for a desire list of true vertical depth values

	Parameters
	----------
	well : str
	  Selected well
	TVD_array : list
	  List of true vertical depth values

	Returns
	-------
	array
	  MD : list of measure depth values
	  
	Attention
	---------
	The input information comes from the files input/ubication.csv and  input/survey/{well}_MD.dat.

	Note
	----
	A linear regression is used.

	Examples
	--------
	>>>TVD_to_MD_array('WELL-1,data['TVD'])
	"""

	file="../input/survey/%s_MD.dat"%well
	MD,DeltaY,DeltaX=np.loadtxt(file,skiprows=1,unpack=True,delimiter=',')

	reader = csv.DictReader(open("../input/ubication.csv", 'r')) #'rb'
	dict_ubication={}
	for line in reader:

		dict_ubication[line['well']]=line
	z_0=float(dict_ubication[well]['masl'])
	x_0=float(dict_ubication[well]['east'])
	y_0=float(dict_ubication[well]['north'])

	#Initialize the delta z values
	z_delta=[0 for i in MD]
	x=[0 for i in MD]
	y=[0 for i in MD]
	z=[0 for i in MD]

	#Assuming straight line between points
	for j in range(len(MD)):
		if j==0:
			z_delta[j]=0
		else:
			z_delta[j]=((MD[j]-MD[j-1])**2-(DeltaX[j]-DeltaX[j-1])**2-(DeltaY[j]-DeltaY[j-1])**2)**0.5+z_delta[j-1]
	
	#Convertion delta to absolute
	for j in range(len(MD)):
		z[j]=z_0-z_delta[j]

	#Function of X-Y-Z with MD
	funzmd=interpolate.interp1d(z,MD)

	MD=[]
	for v in TVD_array:
		try:
			MD.append(funzmd(v))
		except ValueError:
			MD.append(np.nan)
	return MD

def remap(data):
	"""It returns an array with the number of values as the reference pressure array 

	Parameters
	----------
	data : dict
	  Contains 'MD_P','P' and 'MD_T' at least. The depths values for P are found base on the depths of MD_T

	Returns
	-------
	dict
	  data : with 'remap_P' cointaning the values of P for the depths listed at MD_T
	"""

	map_function=interpolate.interp1d(data['MD_P'],data['P'])

	p=[]
	for MD in data['MD_T']:
		try:
			p.append(map_function(MD))
		except ValueError:
			p.append(np.nan)

	data['remap_P']=p

	return data

def line_intersect(Ax1, Ay1, Ax2, Ay2, Bx1, By1, Bx2, By2):
	"""Finds the intersection point between two lines

	Parameters
	----------	  
	{letter}{position}{corr} : float
	  Identifies the point A or B, the position (x or y ) and the correlative (1 or 2)

	Returns
	-------
	tuple
	  (x,y), if there is no intersection, None value is returned
	"""

	d = (By2 - By1) * (Ax2 - Ax1) - (Bx2 - Bx1) * (Ay2 - Ay1)
	if d:
		uA = ((Bx2 - Bx1) * (Ay1 - By1) - (By2 - By1) * (Ax1 - Bx1)) / d
		uB = ((Ax2 - Ax1) * (Ay1 - By1) - (Ay2 - Ay1) * (Ax1 - Bx1)) / d
	else:
		return
	if not(0 <= uA <= 1 and 0 <= uB <= 1):
		return
	x = Ax1 + uA * (Ax2 - Ax1)
	y = Ay1 + uA * (Ay2 - Ay1)

	return x, y

def write_well_track(input_dictionary):
	"""It converts every specified well survey using measure depth on true vertical depth survey

	Parameters
	----------	  
	input_dictionary : dictionary
	   Contains list of wells under the keywords 'WELLS', 'MAKE_UP_WELLS' and 'NOT_PRODUCING_WELL'. It also needs to contain the input files path.

	Returns
	-------
	file
	  {pozo}_xyz.csv: on ../input/survey/xyz

	Examples
	--------
	>>> write_well_track(input_dictionary)
	"""

	source_txt=input_dictionary['source_txt']

	types=['WELLS','MAKE_UP_WELLS','NOT_PRODUCING_WELL']
	wells=[]
	for scheme in types:
		try:
			for well in input_dictionary[scheme]:
				wells.append(well)
		except KeyError:
				pass

	for well in wells:
		try:
			well_survey=pd.read_csv(source_txt+'survey/'+well+"_MD.dat")
			file=open(source_txt+'survey/'+well+"_xyz.csv","w")
			string="x,y,z\n"
			file.write(string)
			for index,row in well_survey.iterrows():
				x,y,z=MD_to_TVD(well,row['MeasuredDepth'])
				string="%s,%s,%s\n"%(x,y,z)
				file.write(string)
			file.close()
		except FileNotFoundError:
			sys.exit("There is no MD file for the well %s"%well)

def write_feedzone_position(input_dictionary):
	""" It generates a file on csv format containing the well feedzone position and coordinates.

	Parameters
	----------	  
	input_dictionary : dictionary
	  Contains the path of the input files under the keyword 'sources_txt'

	Returns
	-------
	file
	  well_feezone_xyz: on ../input

	Note
	----
	regeo_mesh uses this file as an input

	Examples
	--------
	>>> write_feedzone_position(source_txt='../input/')
	"""

	source_txt=input_dictionary['source_txt']

	source_file=source_txt+'well_feedzone.csv'
	try:
		feedzones=pd.read_csv(source_file,delimiter=",")	
	except FileNotFoundError:
		sys.exit("There is no MD file for the well %s"%source_file)

	file=open('../input/well_feedzone_xyz.csv','w')

	string="well,MD,x,y,z,type\n"
	file.write(string)
	for index,row in feedzones.iterrows():
		x,y,z=MD_to_TVD(row['well'],row['MD'])
		string="%s,%s,%s,%s,%s,%s\n"%(row['well'],row['MD'],x,y,z,row['type'])
		print(string)
		file.write(string)
	file.close()

def write_feedzone_position_from_dict(wells):
	""" It generates a file on csv format containing the well feedzone position and coordinates from a python dictionary with well information

	Parameters
	----------	  
	wells : dictionary
	  Contains the wells names, feedzone and contribution

	Returns
	-------
	file
	  well_feedzone_xyz: on ../input

	Note
	----
	regeo_mesh uses this file as an input

	Examples
	--------
	>>> wells={'TR-1':{type_w':'OBS','feedzones':{'A':{'MD':600,'contribution':1}}}},
	>>> write_feedzone_position_from_dict(wells)
	"""

	file=open('../input/'+'well_feedzone_xyz.csv','w')

	string="well,MD,x,y,z,type\n"
	file.write(string)
	for well in wells:
		for i in wells[well]['feedzones']:
			x,y,z=MD_to_TVD(well,wells[well]['feedzones'][i]['MD'])
			string="%s,%s,%s,%s,%s,%s\n"%(well,wells[well]['feedzones'][i]['MD'],x,y,z,wells[well]['type_w'])
			file.write(string)
	file.close()

def write_feedzone_from_dict(wells):
	""" It generates a file on csv format containing the well feedzone position and coordinates from a python dictionary with well information

	Parameters
	----------	  
	wells : dictionary
	  Contains the wells names, feedzone and contribution

	Returns
	-------
	file
	  well_feedzone: on ../input

	Note
	----
	regeo_mesh uses this file as an input

	Examples
	--------
	>>> wells={'TR-1':{type_w':'OBS','feedzones':{'A':{'MD':600,'contribution':1}}}},
	>>> write_feedzone_position_from_dict(wells)
	"""

	file=open('../input/'+'well_feedzone.csv','w')
	string="well,MD,contribution,type\n"
	file.write(string)
	for well in wells:
		for i in wells[well]['feedzones']:
			x,y,z=MD_to_TVD(well,wells[well]['feedzones'][i]['MD'])
			string="%s,%s,%s,%s\n"%(well,wells[well]['feedzones'][i]['MD'],wells[well]['feedzones'][i]['contribution'],wells[well]['type_w'])
			file.write(string)
	file.close()

def PT_natural_to_GIS(input_dictionary):
	"""It generates a shapefile containing pressure and temperature real data from every well at every layer elevation

	Parameters
	----------
	input_dictionary : dictionary
	  Contains the infomation of the layer under the keyword 'LAYER' and 'z_ref'.

	Returns
	-------
	file
	  layer_{corr}_masl_{masl}.shp: on ../input/PT/GIS/

	Note
	----
	The primary data comes from the defined formation data

	Examples
	--------
	>>> PT_natural_to_GIS(input_dictionary)
	"""

	wells=[]

	for key in ['WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	layers_info=vertical_layers(input_dictionary)

	layer_dict={}
	for v in range(len(layers_info['middle'])):
		layer_dict[layers_info['middle'][v]]=layers_info['name'][v]

	for elevation in layers_info['middle']:
		w=shapefile.Writer('../input/PT/GIS/layer_%s_masl_%s.shp'%(layer_dict[elevation],elevation))
		w.field('TEMPERATURE','F',decimal=2)
		w.field('PRESSURE','F',decimal=2)
		for well in sorted(wells):
			file_source_name="../input/PT/%s_MDPT.dat"%well
			try:
				data=pd.read_csv(file_source_name)
				fun_T=interpolate.interp1d(data['MD'],data['T'])
				fun_P=interpolate.interp1d(data['MD'],data['P'])
				try:
					masl_MD=TVD_to_MD(well,elevation)
					x,y,z=MD_to_TVD(well,masl_MD)
					if not math.isnan(fun_T(masl_MD)) and not math.isnan(fun_P(masl_MD)) and x != np.nan:
						print(x,fun_T(masl_MD))
						w.point(x,y)
						w.record(float(fun_T(masl_MD)),float(fun_P(masl_MD)))
				except (ValueError,shapefile.ShapefileException) :
					pass

			except (IOError,shapefile.ShapefileException):
				pass
		w.close()

def PT_real_to_json(input_dictionary):
	"""It writes a json file based on the formation data (pressure and temperature) at the level of the defined mesh layers.

	Parameters
	----------	  
	input_dictionary : dictionary
	  Contains the infomation of the layer under the keyword 'LAYER' and 'z_ref'.

	Returns
	-------
	file
	  PT_real_json: en direccion ../input/PT/PT_real_json.txt

	Examples
	--------
	>>> PT_real_to_json(input_dictionary)
	"""

	wells=[]

	for key in ['WELLS','NOT_PRODUCING_WELL']:
		try:
			for well in input_dictionary[key]:
				wells.append(well)
		except KeyError:
			pass

	layers_info=vertical_layers(input_dictionary)

	layer_dict={}
	for v in range(len(layers_info['middle'])):
		layer_dict[layers_info['middle'][v]]=layers_info['name'][v]

	eleme_dict={}

	for well in sorted(wells):
		file_source_name="../input/PT/%s_MDPT.dat"%well
		try:
			data=pd.read_csv(file_source_name)
			fun_T=interpolate.interp1d(data['MD'],data['T'])
			fun_P=interpolate.interp1d(data['MD'],data['P'])
			for masl in layers_info['middle']:
				masl_MD=TVD_to_MD(well,masl)
				try:
					if not np.isnan(masl_MD):
						x,y,z=MD_to_TVD(well,masl_MD)
						eleme_dict["%s-%s"%(well,layer_dict[masl])]=[float(x),float(y),masl,float(fun_T(masl_MD)),float(fun_P(masl_MD))]
				except ValueError:
					pass
		except IOError:
			pass

	with open("../input/PT/PT_real_json.txt",'w') as json_file:
		  json.dump(eleme_dict, json_file,sort_keys=True, indent=1)

def P_from_T_TVD(T_array,TVD_array,water_level_TVD,Pmin):
	"""It generates a pressure log based on real temperature measurements and a hydrostatic level

	Parameters
	----------
	T_array : array
	  Temperature values
	TVD_array : array
	  TVD values associate with every temperature record
	water_level_TVD : float
	  Hydrostatic level on TVD value

	Returns
	-------
	array
	  P: pressure values

	Examples
	--------
	>>> P_from_T_TVD(T_array,TVD_array,water_level_TVD,Pmin)
	"""

	P=[]
	g=formats.formats_t2['PARAMETERS']['GF']
	for n in range(len(TVD_array)):
		if TVD_array[n]<=water_level_TVD:
			if n==0:
				P.append(Pmin)
			else:
				try:
					C0=IAPWS97(T=(T_array[n]+273.15),x=0)
					P.append((water_level_TVD-TVD_array[n])*g*C0.rho/1E5+Pmin)
				except NotImplementedError:
					P.append(P[n-1])
		else:
			P.append(Pmin)
	return P

def survey_to_GIS(input_dictionary):
	"""It writes a shapefile (line type) from the wells surveys

	Parameters
	----------	  
	input_dictionary : dictionary
	  Dictionary contaning the path and name of database on keyword 'db_path'.

	Returns
	-------
	file
	  well_survey: on the path ../mesh/GIS

	Examples
	--------
	>>> survey_to_GIS(input_dictionary)
	"""

	db_path=input_dictionary['db_path']
	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	wells_data=pd.read_sql_query("SELECT * FROM survey ORDER BY well DESC;",conn)
	wells=wells_data.well.unique()

	if len(wells)!=0:
		w=shapefile.Writer('../mesh/GIS/well_survey')
		w.field('WELL', 'C', size=10)
		for well in wells:
			data = wells_data.loc[wells_data['well'] == well]
			points=np.asarray(MD_to_TVD(well,data['MeasuredDepth']))
			points=np.transpose(points)
			w.linez([points.tolist()])
			w.record(well)
		w.close()
	else:
		sys.exit("There is no well survey store on the database")

def feedzone_to_GIS(input_dictionary):
	"""It writes a shapefile (point type) from the wells feedzones position

	Parameters
	----------	  
	input_dictionary : dictionary
	  Dictionary contaning the path and name of database on keyword 'db_path'.

	Returns
	-------
	file
	  well_survey: on the path ../mesh/GIS

	Examples
	--------
	>>> feedzone_to_GIS(input_dictionary)
	"""


	db_path=input_dictionary['db_path']
	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	wells_data=pd.read_sql_query("SELECT * FROM wellfeedzone ORDER BY well DESC;",conn)
	wells=wells_data.well.unique()

	wells_info=pd.read_sql_query("SELECT * FROM wells ORDER BY well DESC;",conn)


	if len(wells)!=0:
		w=shapefile.Writer('../mesh/GIS/well_feedzone')
		w.field('WELL', 'C', size=10)
		w.field('TYPE', 'C', size=10)
		w.field('CONTRIBUTION', 'F', decimal=2)
		for i, row in wells_data.iterrows():
			x,y,z=MD_to_TVD(row['well'],row['MeasuredDepth'])
			w.pointz(x,y,z)
			w.record(row['well'],wells_info.loc[wells_info['well']==row['well'],'type'].values[0],row['contribution'])
		w.close()
	else:
		sys.exit("There is no well survey store on the database")


from T2GEORES import writer as writer
from T2GEORES import gener as t2g
import os
import subprocess
import shutil

def incon_replace(incon_file,blocks,incon_file_len):
	"""It rewrite the incon file without the porosity, depending if it comes from .sav file or INCON generated with T2GEORES

	Parameters
	----------
	incon_file : str
	  incon file to be rearange
	blocks: str
	  Declares if the file has the block information on even or odd lines
	incon_file_len: int
	  Length of incon file

	Returns
	-------
	str
	  string: formatted string with incon data
	  
	Attention
	---------
	The function is called by incon_to_t2()

	"""
	string=""
	for index, incon_line in enumerate(incon_file):
		if '+++' in incon_line:
			minus = 3
		else: 
			minus = 2
		if blocks=='even' and index!=0 and index<=incon_file_len-minus:
			if index%2==1:
				string+=incon_line[0:5]+'\n'
			else:
				string+=incon_line
		if blocks=='odd' :
			if index%2!=1:
				string+=incon_line
			else:
				string+=incon_line
	return string

def incon_delete():
	"""It reads the TOUGH2 file assuming it is on ../model/t2/ and erases the INCON section

	Returns
	-------
	file
	  t2: file without INCON information
	  
	Attention
	---------
	There is no need of input argument

	Examples
	--------
	>>> incon_delete()
	"""
	final_t2=""
	incon_section=False
	input_fi_file="../model/t2/t2"
	if os.path.isfile(input_fi_file):
		t2_file=open(input_fi_file, "r")
		for t2_line in t2_file:
			if 'INCON' in t2_line:
				incon_section=True
			elif 'ELEME' in t2_line:
				incon_section=False
			if not incon_section:
				final_t2+=t2_line
		t2_file.close()
		t2_file_out=open(input_fi_file, "w")
		t2_file_out.write(final_t2)
		t2_file_out.close()	
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)

def incon_to_t2(input_dictionary, file_name = None):
	"""Adds the INCON block to the TOUGH2 input file

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary contanining the information of the source of incon with keyword 'incon_state' can take the value of 'current' or 'init'.

	Returns
	-------
	file
	  t2: modified file on  ../model/t2/t2 

	Examples
	--------
	>>> incon_to_t2(input_dictionary)
	"""

	incon_state=input_dictionary['incon_state']

	if incon_state=='current':
		input_incon="../model/t2/%s"%file_name
		blocks='even'
		incon_file_len=len(open(input_incon).readlines())
	elif incon_state=='init':
		input_incon="../model/t2/sources/INCON"
		blocks='odd'
		incon_file_len=len(open(input_incon).readlines())
	elif incon_state=='steinar':
		input_incon="../model/t2/sources/STE_INCON"
		blocks='odd'
		incon_file_len=len(open(input_incon).readlines())
	elif incon_state=='init_mod':
		input_incon="../model/t2/sources/INCON_MOD"
		blocks='odd'
		incon_file_len=len(open(input_incon).readlines())

	input_fi_file="../model/t2/t2"

	if os.path.isfile(input_fi_file):
		if os.path.isfile(input_incon):
			incon_file=open(input_incon, "r")
			t2_file=open(input_fi_file, "r")
			if 'INCON' not in open(input_fi_file).read():
				final_t2=""
				for t2_line in t2_file:
					if 'ELEME' not in t2_line:
						final_t2+=t2_line
					elif 'ELEME' in t2_line:
						final_t2+='INCON\n'
						final_t2+=incon_replace(incon_file,blocks,incon_file_len)
						final_t2+='\nELEME----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
			else:
				final_t2=""
				incon_section=False
				for t2_line in t2_file:
					if 'INCON' in t2_line:
						incon_section=True
					elif 'ELEME' in t2_line:
						incon_section=False
						final_t2+='INCON\n'
						final_t2+=incon_replace(incon_file,blocks,incon_file_len)
						final_t2+='\n'
					if not incon_section:
						final_t2+=t2_line

			incon_file.close()
			t2_file.close()
			t2_file_out=open(input_fi_file, "w")
			t2_file_out.write(final_t2)
			t2_file_out.close()
		else:
			sys.exit("The file %s or directory do not exist"%input_fi_file)
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)

def update_gen(input_dictionary):
	"""It updates the file the TOUGH2 input file based on the previous updates of all files on ../model/t2/sources/GENER_*

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary contaning the type of desire run 'production' or 'natural' written on the keyword 'TYPE_RUN'

	Returns
	-------
	file
	  t2: file with the modified version of GENER

	Note
	----
	Some print sentences are expected depending on the availability of certain files

	Examples
	--------
	>>> update_gen(input_dictionary)
	"""

	type_run=input_dictionary['TYPE_RUN']

	final_t2=""
	incon_section=False
	input_fi_file="../model/t2/t2"
	sources_file='../model/t2/sources/GENER_SOURCES'
	well_sources_file='../model/t2/sources/GENER_PROD'
	makeup_well_sources_file='../model/t2/sources/GENER_MAKEUP'
	t2_ver=float(input_dictionary['VERSION'][0:3])

	if os.path.isfile(input_fi_file):
		if os.path.isfile(sources_file):
			if 'GENER' not in open(input_fi_file).read():

				t2_string=""
				t2_file=open(input_fi_file, "r")
				for t2_line in t2_file:
					if 'ENDCY' not in t2_line:
						t2_string+=t2_line
					else:
						pass

				if t2_ver<7:
					t2_string+='GENER----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
				else:
					t2_string+='GENER D--1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'

				
				gener_source_file=open(sources_file, "r")
				for sources in gener_source_file:
					t2_string+=sources
				gener_source_file.close()
			
				if type_run=='production':
					if os.path.isfile(well_sources_file):
						well_source_file=open(well_sources_file, "r")
						for sources in well_source_file:
							t2_string+=sources
						well_source_file.close()

					else:
						print("The file %s or directory do not exist"%well_sources_file)

					if os.path.isfile(makeup_well_sources_file):
						well_makeup_source_file=open(makeup_well_sources_file, "r")
						for sources in well_makeup_source_file:
							t2_string+=sources
						well_makeup_source_file.close()
					else:
						print("The file %s or directory do not exist"%well_sources_file)

				t2_string+='\n'
				t2_string+='ENDCY----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
				t2_file.close()
				

				t2_file_out=open(input_fi_file, "w")
				t2_file_out.write(t2_string)
				t2_file_out.close()	

			else:
				t2_string=""
				t2_file=open(input_fi_file, "r")
				for t2_line in t2_file:
					if 'GENER' not in t2_line:
						t2_string+=t2_line
					else:

						if t2_ver<7:
							t2_string+='GENER----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
						else:
							t2_string+='GENER D--1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'

							t2_string+="AA110DUM00                   3     MASS                      10.e5\n"
							t2_string+="           -infinity       0.0\n"
							t2_string+="%s       0.0\n"%(format(input_dictionary['ref_date'].strftime("%Y-%m-%d_00:00:00"),'>20s'))
							t2_string+="            infinity       0.0\n"

						gener_source_file=open(sources_file, "r")
						for sources in gener_source_file:
							t2_string+=sources

						if type_run=='production':
							if os.path.isfile(well_sources_file):
								well_source_file=open(well_sources_file, "r")
								for sources in well_source_file:
									t2_string+=sources
								well_source_file.close()
							else:
								print("The file %s or directory do not exist"%well_sources_file)

						if os.path.isfile(makeup_well_sources_file):
							well_makeup_source_file=open(makeup_well_sources_file, "r")
							for sources in well_makeup_source_file:
								t2_string+=sources
							well_makeup_source_file.close()
						else:
							print("The file %s or directory do not exist"%well_sources_file)

						break
					
				t2_string+='\nENDCY----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n'
				t2_file.close()

				t2_file_out=open(input_fi_file, "w")
				t2_file_out.write(t2_string)
				t2_file_out.close()	
		else:
			sys.exit("The file %s or directory do not exist"%sources_file)
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)

def update_rock_distribution(input_dictionary):
	"""It updates the file the TOUGH2 input file based on the new rock distribution

	Parameters
	----------
	input_dictionary : dictionary
	  Dictionary contaning the information where the rock distribution comes from

	Returns
	-------
	file
	  t2: file with the modified version of ELEME and CONNE

	Examples
	--------
	>>> update_rock_distribution(input_dictionary)
	"""

	writer.CONNE_from_steinar_to_t2()
	writer.merge_ELEME_and_in_to_t2()

	t2_string=""
	input_fi_file="../model/t2/t2"

	if os.path.isfile(input_fi_file):
		t2_file=open(input_fi_file, "r")
		ELEME_line=1E50
		add_gener=False
		add_ELEME=False
		for i,t2_line in enumerate(t2_file):
			if 'ELEME' not in t2_line and ELEME_line>i:
				t2_string+=t2_line

			if 'ELEME' in t2_line:
				add_ELEME=True
				ELEME_line=0

			if 'GENER D' in t2_line:
				add_gener=True

			if add_gener:
				t2_string+=t2_line

			if add_ELEME:
				t2_string+=writer.ELEME_adder(input_dictionary)
				t2_string+=writer.CONNE_adder(input_dictionary)
				add_ELEME=False

		t2_file.close()
		t2_file_out=open(input_fi_file, "w")
		t2_file_out.write(t2_string)
		t2_file_out.close()
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)

def copy_folder(src,dest):
	"""It copies all the files on a folder to another folder

	Parameters
	----------
	src : str
	  Source directory
	dest : str
	  Output directory

	Examples
	--------
	>>> copy_folder('../model/t2','../model/t2/prev')
	"""
	src_files = os.listdir(src)
	for file_name in src_files:
		full_file_name = os.path.join(src, file_name)
		if os.path.isfile(full_file_name):
			try:
				shutil.copy(full_file_name, dest)
			except shutil.Error:
				print("%s and %s didnt change"%(full_file_name,dest))

def create_prev():
	"""It creates a copy of several folder for further comparisson

	Note
	----
	The folders to copy are: 
	'../model/t2'
	'../output/PT/images'
	'../output/PT/json'
	'../output/PT/txt'
	'../output/PT/evol'
	'../output/mh'
	All of them contains a subfolder prev where the data is stored

	Examples
	--------
	>>> create_prev()
	"""

	#Copy t2 output
	src='../model/t2'
	dest= '../model/t2/prev'
	copy_folder(src,dest)

	#Copy prev images output
	src='../output/PT/images'
	dest='../output/PT/images/prev'
	copy_folder(src,dest)

	#Copy prev json output
	src='../output/PT/json'
	dest='../output/PT/json/prev'
	copy_folder(src,dest)

	#Copy prev txt output
	src='../output/PT/txt'
	dest='../output/PT/txt/prev'
	copy_folder(src,dest)

	#Copy prev txt output
	src='../output/PT/evol'
	dest='../output/PT/evol/prev'
	copy_folder(src,dest)

	#Copy prev txt output
	src='../output/mh/txt'
	dest='../output/mh/txt/prev'
	copy_folder(src,dest)

def run(input_dictionary):
	"""It runs the TOUGH2 input file

	Parameters
	----------
	input_dictionary : dictionary
	  A dictionary with the EOS used on the model

	Returns
	-------
	files
	  various: containing the output from modelling 
	"""

	EOS=input_dictionary['EOS']
	version=input_dictionary['VERSION']
	current=os.path.abspath(os.getcwd())
	t2_input_file='%s/model/t2/t2'%current.replace('/scripts','')
	it2_input_file='%s/model/t2/fit2'%current.replace('/scripts','')
	shutil.copyfile(t2_input_file, current+'/t2')
	shutil.copyfile(it2_input_file, current+'/fit2')

	if not input_dictionary['IT2_alias']:
		subprocess.run(["itough2",'-v',version,'fit2','t2',str(EOS)])
	else:
		subprocess.run([input_dictionary['IT2_alias'],'fit2','t2',str(EOS)])

	target_dir = current.replace('/scripts','/model/t2')
	file_names = os.listdir(current)

	for file_name in file_names:
		if 't2.' in file_name or 'fit2' in file_name:
			try:
				shutil.move(os.path.join(current, file_name),os.path.join(target_dir, file_name) )
			except shutil.Error:
				print("File already exist %s"%file_name)

def rock_update(input_dictionary):
	"""
	It updates the rock type on the TOUGH2 file
	Parameters
	----------
	input_dictionary : dictionary
		TOUGH2 file name
	"""

	t2_string = ""
	input_fi_file = "../model/t2/%s"%input_dictionary['TOUGH2_file']
	if os.path.isfile(input_fi_file):
		t2_file=open(input_fi_file, "r")
		ELEME_line = 1E50
		add_param = False
		add_rocks = False
		for i,t2_line in enumerate(t2_file):

			if 'ROCKS' not in t2_line and ELEME_line>i:
				t2_string+=t2_line

			if 'ROCKS' in t2_line:
				add_rocks=True
				ELEME_line=0

			if 'PARAM' in t2_line or 'START' in t2_line:
				add_param = True

			if add_param:
				t2_string += t2_line

			if add_rocks:
				t2_string += writer.ROCKS_writer(input_dictionary)
				add_rocks = False

		t2_file.close()
		t2_file_out = open(input_fi_file, "w")
		t2_file_out.write(t2_string)
		t2_file_out.close()
	else:
		sys.exit("The file %s or directory do not exist"%input_fi_file)